#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import sys

from chipwhisperer.common.utils.parameter import Parameterized, setupSetParam
from chipwhisperer.common.utils import util
import numpy as np

#The following is placed here for backwards compatability with imports expected the getHW function
_HW = [0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 3, 3,
       4, 2, 3, 3, 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4,
       4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 1, 2, 2, 3, 2,
       3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5,
       4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4,
       5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3,
       3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2,
       3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
       4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5,
       6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5,
       5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6,
       7, 7, 8]

def getHW(byte):
    return _HW[byte]


class ModelsBase(Parameterized):
    _name = 'Crypto Model'

    hwModels = util.DictType()
    hwModels_toStr = []

    ##Generate this table with:
    #HW = []
    #for n in range(0, 256):
    #    HW = HW + [bin(n).count("1")]
    HW = [0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 3, 3,
          4, 2, 3, 3, 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4,
          4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 1, 2, 2, 3, 2,
          3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5,
          4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4,
          5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3,
          3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2,
          3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6,
          4, 5, 5, 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5,
          6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5,
          5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6,
          7, 7, 8]

    def __init__(self, numSubKeys=None, permPerSubkey=None, model=None):
        self.sigParametersChanged = util.Signal()
        self.numSubKeys = numSubKeys
        self.permPerSubkey = permPerSubkey
        self.numRoundKeys = 0
        self.model = model
        #Update interal models (if required)
        self._updateHwModel()

        if model.name in self.hwModels:
            if self.hwModels[model.name] != model:
                self.hwModels[str(model)] = model
        else:
            self.hwModels[model.name] = model

        self.getParams().addChildren([
            {'name':'Hardware Model', 'type':'list', 'values':self.hwModels, 'get':self.getHwModel, 'set':self.setHwModel, 'addToList':True}
        ])

        if numSubKeys:
            self.getParams().addChildren([{'name':'Number of SubKeys', 'type':'int', 'get':self.getNumSubKeys, 'readonly':True}])

        if permPerSubkey:
            self.getParams().addChildren([{'name':'Number of Permutations', 'type':'int', 'get':self.getPermPerSubkey, 'readonly':True}])

    @classmethod
    def getHW(cls, n):
        return cls.HW[n]

    def processKnownKey(self, inpkey):
        return inpkey

    def leakage(self, pt, ct, guess, bnum, state):
        pass

    def getNumSubKeys(self):
        return self.numSubKeys

    def getPermPerSubkey(self):
        return self.permPerSubkey

    def getNumRoundKeys(self):
        return self.numRoundKeys

    def getHwModel(self):
        return self.model

    def keyScheduleRounds(self, inputkey, inputround, desiredround, returnSubkeys=True):
        raise NotImplementedError()

    def getRoundKeys(self, key, iniRound):
        ret = []
        for targetRound in range(0, self.numRoundKeys+1):
            ret.append(self.keyScheduleRounds(key, iniRound, targetRound, returnSubkeys=False))
        return ret

    @setupSetParam("Hardware Model")
    def setHwModel(self, model, kwargs=None):
        self.model = model
        self.model_kwargs = kwargs
        self._updateHwModel()
        self.sigParametersChanged.emit()

    def _updateHwModel(self):
        """" Re-implement this to update leakage model """
        pass

    def _VccToGnd(self, var):
        """Convert from number of 1's to number of 0's... used when shunt inserted in GND path"""
        return 8-var



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import copy
import logging
import random

import numpy as np
from chipwhisperer.analyzer.attacks.models.aes.funcs import sbox, inv_sbox
from chipwhisperer.analyzer.attacks.models.AES128_8bit import AES128_8bit
from chipwhisperer.analyzer.attacks.models.aes.key_schedule import keyScheduleRounds
from chipwhisperer.common.utils.parameter import Parameterized
from chipwhisperer.common.utils import util


class PartitionHDLastRound(object):

    sectionName = "Partition Based on HD of Last Round"
    partitionType = "HD AES Last-Round"

    def getNumPartitions(self):
        return 9

    def getPartitionNum(self, trace, tnum):
        key = trace.getKnownKey(tnum)
        ct = trace.getTextout(tnum)

        #Convert from initial key to final-round key, currently
        #this assumes AES
        if len(key) == 16:
            rounds = 10
        else:
            raise ValueError("Need to implement for selected AES")
        key = keyScheduleRounds(key, 0, rounds)

        guess = [0] * 16
        for i in range(0, 16):
            st10 = ct[AES128_8bit.INVSHIFT[i]]
            st9 = inv_sbox(ct[i] ^ key[i])
            guess[i] = AES128_8bit.getHW(st9 ^ st10)
        return guess


class PartitionHWIntermediate(object):

    sectionName = "Partition Based on HW of Intermediate"
    partitionType = "HW AES Intermediate"

    def getNumPartitions(self):
        return 9

    def getPartitionNum(self, trace, tnum):
        key = trace.getKnownKey(tnum)
        text = trace.getTextin(tnum)

        guess = [0] * 16
        for i in range(0, 16):
            guess[i] = AES128_8bit.getHW(sbox(text[i] ^ key[i]))

        return guess


class PartitionEncKey(object):

    sectionName = "Partition Based on Key Value"
    partitionType = "Key Value"

    def getNumPartitions(self):
        return 256

    def getPartitionNum(self, trace, tnum):
        key = trace.getKnownKey(tnum)
        return key


class PartitionRandvsFixed(object):
    """The Rand vs Fixed partition works with the TVLA test to randomly interleave random and fixed plaintexts.

    In this capture mode, half of the plaintexts are random and half have a fixed value (specified in TVLA papers).
    Different fixed plaintexts are used for AES-128, -192, and -256.
    To check which partition a trace is in, we just need to check if the plaintext matches the standard fixed plaintext
    for a given key length.
    """

    sectionName = "Partition Based on Rand vs Fixed "
    partitionType = "Rand vs Fixed"

    def getNumPartitions(self):
        return 2

    def getPartitionNum(self, trace, tnum):
        """Checks if plaintext is the fixed TVLA plaintext for this key length or a random value.

        Returns [1] if fixed and [0] if random.
        """
        klen = len(trace.getKnownKey(tnum))

        pt = trace.getTextin(tnum)
        if klen == 16 and (pt == util.hexStrToByteArray("da 39 a3 ee 5e 6b 4b 0d 32 55 bf ef 95 60 18 90")).all():
            return [1]
        elif klen == 24 and (pt == util.hexStrToByteArray("da 39 a3 ee 5e 6b 4b 0d 32 55 bf ef 95 60 18 88")).all():
            return [1]
        elif klen == 32 and (pt == util.hexStrToByteArray("da 39 a3 ee 5e 6b 4b 0d 32 55 bf ef 95 60 18 95")).all():
            return [1]
        return [0]


class PartitionRandDebug(object):

    sectionName = "Partition Randomly (debug)"
    partitionType = "Randomly (debug)"

    numRand = 2

    def getNumPartitions(self):
        return self.numRand

    def getPartitionNum(self, trace, tnum):
        return [random.randint(0, self.numRand - 1)]


class Partition(Parameterized):
    """
    Base Class for all partioning modules
    """
    # traceDone = Signal(int)

    _description = "Partition traces based on some method."

    attrDictPartition = {
                "sectionName":"Partition Based on XXXX",
                "moduleName":"Partitions",
                "module":None,
                "values":{
                    "round":{"value":0, "desc":"Round", "changed":False, "definesunique":True},
                    "filename":{"value":None, "desc":"Partition File", "changed":False, "headerLabel":"Partition Data"},
                    },
                }

    supportedMethods = [PartitionRandvsFixed, PartitionEncKey, PartitionRandDebug, PartitionHWIntermediate, PartitionHDLastRound]

    def __init__(self):
        self.setPartMethod(PartitionRandvsFixed)
        self.partDataCache = None

    def setPartMethod(self, method):
        self.partMethodClass = method
        self.partMethod = method()
        self.attrDictPartition["sectionName"] = self.partMethod.sectionName
        self.attrDictPartition["moduleName"] = self.partMethod.__class__.__name__

    def init(self):
        """Do any initilization required once all traces are loaded"""
        pass

    def createBlankTable(self, num_keys, num_parts):
        # Create storage for partition information
        partitionTable = []
        #for j in range(0, len(t.getKnownKey())):
        for j in range(0, num_keys):
            partitionTable.append([])
            for i in range(0, num_parts):
                partitionTable[j].append([])

        return partitionTable

    def loadPartitions(self, tRange=(0, -1)):
        """Load partitions from trace files, convert to mapped range"""
        start = tRange[0]
        end = tRange[1]

        if end == -1:
            end = self._traces.numTraces()

        # Generate blank partition table
        partitionTable = self.createBlankTable(self._traces.getSegment(start))
        logging.info(np.shape(partitionTable))

        tnum = start
        while tnum < end:
            t = self._traces.getSegment(tnum)
            # Discover where this trace starts & ends
            tmapstart = t.mappedRange[0]
            tmapend = t.mappedRange[1]
            tmapend = min(tmapend, end)

            partcfg = t.getAuxDataConfig(self.attrDictPartition)
            # print partcfg
            # print partcfg["filename"]
            partdata = t.loadAuxData(partcfg["filename"])

            # Merge tables now - better way to do this?
            for j in range(0, len(self.partMethod.getPartitionNum(t, 0))):
                for i in range(0, self.partMethod.getNumPartitions()):
                    partitionTable[j][i] = partitionTable[j][i] + partdata[j][i]

            # print tmapstart

            # Next trace round
            tnum = tmapend + 1

        return partitionTable

    def getPartitionData(self):
        return self.partDataCache

    def generatePartitions(self, partitionClass=None, saveFile=False, loadFile=False, tRange=(0, -1)):
        """
        Generate partitions, using previously setup setTraceManager & partition class, or if they are passed as
        arguments will update the class data
        """
        if partitionClass:
            self.setPartMethod(partitionClass)

        partitionTable = None

        if loadFile:
            partitionTable = self.loadPartitions(tRange)
            raise AttributeError("WARNING: loadPartitions broken for split files, disabled for now")

        start = tRange[0]
        end = tRange[1]

        num_keys=len(self.partMethod.getPartitionNum(self._traces.getSegment(start), 0))
        num_parts=self.partMethod.getNumPartitions()

        if partitionTable is None:
            partitionTable = self.createBlankTable(num_keys, num_parts)

            if end == -1:
                end = self._traces.numTraces()

            tnum = start
            while tnum < end:
                t = self._traces.getSegment(tnum)
                # Discover where this trace starts & ends
                tmapstart = t.mappedRange[0]
                tmapend = t.mappedRange[1]
                
                partitionTableTemp = self.createBlankTable(num_keys, num_parts)

                for tnum in range(tmapstart, tmapend + 1):
                    # Check each trace, write partition number
                    partNum = self.partMethod.getPartitionNum(t, tnum - tmapstart)
                    for i, pn in enumerate(partNum):
                        partitionTable[i][pn].append(tnum)
                        partitionTableTemp[i][pn].append(tnum - tmapstart)

                    # self.traceDone.emit(tnum)

                if saveFile:
                    # Save partition table, reference it in config file
                    newCfgDict = copy.deepcopy(self.attrDictPartition)
                    updatedDict = t.addAuxDataConfig(newCfgDict)
                    t.saveAuxData(partitionTableTemp, updatedDict)

                # Debug - Dump Table
                # for t in range(0, self.partMethod.getNumPartitions()):
                #    print "Traces in %d:" % t
                #    print "  ",
                #    print partitionTable[0][t]

                tnum = tmapend + 1

        self.partDataCache = partitionTable
        return partitionTable

    def setTraceSource(self, traces):
        self._traces = traces



"""
Package containing all of the scope types that the ChipWhisperer API can connect to:

Scopes:

   * OpenADC- CWLite and CWPro

   * CWNano - CWNano

   * PicoScope - PicoScope (old, untested)

   * VisaScope - VisaScope (requires Visa module, old, untested)

"""
from chipwhisperer.capture.scopes.OpenADC import OpenADC
from .cwnano import CWNano



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
import logging
import os
import re
import sys
from datetime import datetime
import zipfile
import numpy as np

from chipwhisperer.common.api.TraceManager import TraceManager
from chipwhisperer.common.api.settings import Settings
from chipwhisperer.common.utils.parameter import Parameter, Parameterized, setupSetParam
from chipwhisperer.common.utils import util
from chipwhisperer.common.traces.TraceContainerNative import TraceContainerNative
import copy
from chipwhisperer.common.traces import Trace
import shutil

try:
    from configobj import ConfigObj  # import the module
except ImportError:
    print("ERROR: configobj (https://pypi.python.org/pypi/configobj/) is required for this program")
    sys.exit()

__author__ = "Colin O'Flynn"


def ensure_cwp_extension(path):
    """Ensures that a file path has the '.cwp' extension.

    This can be used to allow specifying project names without
    having to include the '.cwp' extension needed for some of
    the project functionality to work.

    If the file already has the right extension another one
    is not appended.

    Args:
        path (str): A path like string.

    Returns:
        (str) A path that ends with '.cwp'.
    """
    root, ext = os.path.splitext(path)
    if ext != '.cwp':
        return ''.join([root, ext, '.cwp'])
    else:
        return path


class ConfigObjProj(ConfigObj):
    """
    Extends ConfigObj to add a callback feature when something is written, used
    to determine when the project file becomes 'dirty'.
    """

    def __init__(self, callback=None, *args, **kwargs):
        if callback is not None:
            self._callback = util.WeakMethod(callback)
        else:
            self._callback = None

        super(ConfigObjProj, self).__init__(*args, **kwargs)

    def __setitem__(self, key, value, unrepr=False):
        super(ConfigObjProj, self).__setitem__(key, value, unrepr)
        if self._callback:
            self._callback(key)


class Project(Parameterized):
    """Class describing an open ChipWhisperer project.

    Basic capture usage::

        import chipwhisperer as cw
        proj = cw.create_project("project")
        trace = cw.Trace(trace_data, plaintext, ciphertext, key)
        proj.traces.append(trace)
        proj.save()

    Basic analyzer usage::

        import chipwhisperer as cw
        import chipwhisperer.analyzer as cwa
        proj = cw.open_project("project")
        attack = cwa.cpa(proj)
        #run attack

    Use a trace_manager when analyzing traces, since that allows analyzer to
    work with multiple trace segments.

      *  :attr:`project.location <.Project.location>`
      *  :attr:`project.waves <.Project.waves>`
      *  :attr:`project.textins <.Project.textins>`
      *  :attr:`project.textouts <.Project.textouts>`
      *  :attr:`project.keys <.Project.keys>`
      *  :meth:`project.get_filename <.Project.get_filename>`
      *  :meth:`project.trace_manager <.Project.trace_manager>`
      *  :meth:`project.save <.Project.save>`
      *  :meth:`project.export <.Project.export>`
    """
    untitledFileName = os.path.normpath(os.path.join(Settings().value("project-home-dir"), "tmp", "default.cwp"))

    def __init__(self, prog_name="ChipWhisperer", prog_ver=""):
        self.valid_traces = None
        self._trace_format = None

        self.params = Parameter(name="Project Settings", type="group")
        self.params.addChildren([
            {'name': 'Trace Format', 'type': 'list', 'values': self.valid_traces, 'get': self.get_trace_format, 'set': self.set_trace_format},
        ])

        #self.findParam("Trace Format").setValue(TraceContainerNative(project=self), addToList=True)
        self._trace_format = TraceContainerNative(project=self)

        #self.traceParam = Parameter(name="Trace Settings", type='group', addLoadSave=True).register()
        #self.params.getChild('Trace Format').stealDynamicParameters(self.traceParam)

        self.sigFilenameChanged = util.Signal()
        self.sigStatusChanged = util.Signal()
        self.dirty = util.Observable(True)

        self.settingsDict = {'Project Name':"Untitled", 'Project File Version':"1.00", 'Project Author':"Unknown"}
        self.datadirectory = ""
        self.config = ConfigObjProj(callback=self.configObjChanged)
        self._traceManager = TraceManager().register()
        self._traceManager.dirty.connect(self.__dirtyCallback)
        self.setFilename(Project.untitledFileName)

        self.setProgramName(prog_name)
        self.setProgramVersion(prog_ver)

        self._segments = Segments(self)
        self._traces = Traces(self)
        self._keys = IndividualIterable(self._traceManager.get_known_key, self._traceManager.num_traces)
        self._textins = IndividualIterable(self._traceManager.get_textin, self._traceManager.num_traces)
        self._textouts = IndividualIterable(self._traceManager.get_textout, self._traceManager.num_traces)
        self._waves = IndividualIterable(self._traceManager.get_trace, self._traceManager.num_traces)

        if __debug__:
            logging.debug('Created: ' + str(self))

    def append_segment(self, segment):
        """ Adds a new trace segment to the project

        Args:
            segment (TraceContainer): Trace segment to add to project
        """
        self._traceManager.appendSegment(copy.copy(segment))

    appendSegment = append_segment

    def new_segment(self):
        """ Returns the __class__() of the trace container used to store traces
        in the project

        You probably want get_new_trace_segment()

        Returns:
            __class()__ of the current trace container
        """
        return self._trace_format.__class__()

    newSegment = new_segment

    def get_trace_format(self):
        """ Gets the TraceContainer used to store traces

        Returns:
            The trace container used in the project
        """
        return self._trace_format

    getTraceFormat = get_trace_format

    def get_new_trace_segment(self):
        """Return a new TraceContainer object for the specified format.

        Once you're done working with this segment, you can readd it to the
        project with append_segment()

        Returns:
            a new TraceContainer object
        """
        tmp = copy.copy(self._trace_format)
        tmp.clear()
        starttime = datetime.now()
        prefix = starttime.strftime('%Y.%m.%d-%H.%M.%S') + "_"
        tmp.config.setConfigFilename(os.path.join(self.datadirectory, "traces", "config_" + prefix + ".cfg"))
        tmp.config.setAttr("prefix", prefix)
        tmp.config.setAttr("date", starttime.strftime('%Y-%m-%d %H:%M:%S'))
        return tmp

    getNewTraceSegment = get_new_trace_segment

    @setupSetParam("Trace Format")
    def set_trace_format(self, trace_format):
        self._trace_format = trace_format

    setTraceFormat = util.camel_case_deprecated(set_trace_format)

    def __dirtyCallback(self):
        self.dirty.setValue(self._traceManager.dirty.value() or self.dirty.value())

    def configObjChanged(self, key):
        self.dirty.setValue(True)

    def isUntitled(self):
        return self.filename == Project.untitledFileName

    def trace_manager(self):
        """ Gets the trace manager for the project

        Returns:
            The trace manager for the project
        """
        return self._traceManager

    traceManager = util.camel_case_deprecated(trace_manager)

    def setProgramName(self, name):
        self.settingsDict['Program Name']=name

    def setProgramVersion(self, ver):
        self.settingsDict['Program Version']=ver

    def setAuthor(self, author):
        self.settingsDict['Project Author']=author

    def setProjectName(self, name):
        self.settingsDict['Project Name']=name

    def setFileVersion(self, ver):
        self.settingsDict['Project File Version']=ver


    def get_filename(self):
        """Gets the filename associated with the project.

        Returns:
            Filename of the project config file ending with ".cwp".
        """
        return self.filename

    getFilename = util.camel_case_deprecated(get_filename)

    def setFilename(self, f):
        self.filename = f
        self.config.filename = f
        self.datadirectory = os.path.splitext(self.filename)[0] + "_data"
        self.createDataDirectory()
        self.sigStatusChanged.emit()

    def createDataDirectory(self):
        # Check if data-directory exists?
        if not os.path.isdir(self.datadirectory):
            os.makedirs(self.datadirectory)

        # Make trace storage directory too
        if not os.path.isdir(os.path.join(self.datadirectory, 'traces')):
            os.mkdir(os.path.join(self.datadirectory, 'traces'))

        # Make analysis storage directory
        if not os.path.isdir(os.path.join(self.datadirectory, 'analysis')):
            os.mkdir(os.path.join(self.datadirectory, 'analysis'))

        # Make glitchresults storage directory
        if not os.path.isdir(os.path.join(self.datadirectory, 'glitchresults')):
            os.mkdir(os.path.join(self.datadirectory, 'glitchresults'))

    def load(self, f = None):
        if f is not None:
            self.setFilename(os.path.abspath(os.path.expanduser(f)))

        if not os.path.isfile(self.filename):
            raise IOError("File " + self.filename + " does not exist or is not a file")

        self.config = ConfigObjProj(infile=self.filename, callback=self.configObjChanged)
        self._traceManager.loadProject(self.filename)
        self.dirty.setValue(False)

    def getDataFilepath(self, filename, subdirectory='analysis'):
        datadir = os.path.join(self.datadirectory, subdirectory)
        fname = os.path.join(datadir, filename)
        relfname = os.path.relpath(fname, os.path.split(self.config.filename)[0])
        fname = os.path.normpath(fname)
        relfname = os.path.normpath(relfname)
        return {"abs":fname, "rel":relfname}

    def convertDataFilepathAbs(self, relativepath):
        return os.path.join(os.path.split(self.filename)[0], relativepath)

    def checkDataConfig(self, config, requiredSettings):
        """Check a configuration section for various settings. Don't use."""
        requiredSettings = util.convert_to_str(requiredSettings)
        config = util.convert_to_str(config)
        return set(requiredSettings.items()).issubset(set(config.items()))

    def getDataConfig(self, sectionName="Aux Data", subsectionName=None, requiredSettings=None):
        """ Don't use.
        Get all configuration sections of data type given in
        __init__() call, and also matching the given sectionName.
        e.g. if dataName='Aux Data' and sectionName='Frequency',
        this would return a list of all sections of the type
        'Aux Data NNNN - Frequency'.
        """
        sections = []

        # Get all section names
        for sname in list(self.config.keys()):
            # Find if starts with 'Aux Data'
            if sname.startswith(sectionName):
                # print "Found %s" % sname
                # Find if module name matches if applicable
                if subsectionName is None or sname.endswith(subsectionName):
                    # print "Found %s" % sname

                    if requiredSettings is None:
                        sections.append(self.config[sname])
                    else:
                        self.checkDataConfig(self.config[sname], requiredSettings)

        return sections

    def addDataConfig(self, settings=None, sectionName="Aux Data", subsectionName=None):
        # Check configuration file to find incrementing number. Don't use
        maxNumber = 0
        for sname in list(self.config.keys()):
            # Find if starts with 'Aux Data'
            if sname.startswith(sectionName):
                maxNumber = int(re.findall(r'\d+', sname)[0]) + 1

        cfgSectionName = "%s %04d" % (sectionName, maxNumber)
        if subsectionName:
            cfgSectionName += " - %s" % subsectionName

        # Generate the configuration section
        self.config[cfgSectionName] = {}

        if settings is not None:
            for k in list(settings.keys()):
                self.config[cfgSectionName][k] = settings[k]

        return self.config[cfgSectionName]

    def saveAllSettings(self, fname=None, onlyVisibles=False):
        """ Save registered parameters to a file, so it can be loaded again latter. Don't use."""
        if fname is None:
            fname = os.path.join(self.datadirectory, 'settings.cwset')
            logging.info('Saving settings to file: ' + fname)
        Parameter.saveRegistered(fname, onlyVisibles)

    def saveTraceManager(self):
        #Waveform list is Universal across ALL types. Don't use.
        if 'Trace Management' not in self.config:
            self.config['Trace Management'] = {}

        self._traceManager.save_project(self.config, self.filename)

    def save(self):
        """Saves the project.

        Writes the project to the disk. Before this is called your data
        is not saved.
        """
        if self.filename is None:
            return

        self.saveTraceManager()

        #self.config['Waveform List'] = self.config['Waveform List'] + self.waveList

        #Program-Specific Options
        pn = self.settingsDict['Program Name']

        self.config[pn] = {}
        self.config[pn]['General Settings'] =  self.settingsDict

        self.config.write()
        self.sigStatusChanged.emit()
        self.dirty.setValue(False)

    def checkDiff(self):
        """ Don't use.
        Check if there is a difference - returns True if so, and False
        if no changes present. Also updates widget with overview of the
        differences if requested with updateGUI
        """
        self.saveTraceManager()

        disk = util.convert_to_str(ConfigObjProj(infile=self.filename))
        ram = util.convert_to_str(self.config)

    def hasDiffs(self):
        if self.dirty.value(): return True

        #added, removed, changed = self.checkDiff()
        if (len(added) + len(removed) + len(changed)) == 0:
            return False
        return True

    def consolidate(self, keepOriginals = True):
        for indx, t in enumerate(self._traceManager.traceSegments):
            destinationDir = os.path.normpath(os.path.join(self.datadirectory, "traces"))
            config = ConfigObj(t.config.configFilename())
            prefix = config['Trace Config']['prefix']
            tracePath = os.path.normpath(os.path.split(t.config.configFilename())[0])

            if destinationDir == tracePath: continue

            for traceFile in os.listdir(tracePath):
                if traceFile.startswith(prefix):
                    util.copyFile(os.path.normpath(os.path.join(tracePath, traceFile)), destinationDir, keepOriginals)

            util.copyFile(t.config.configFilename(), destinationDir, keepOriginals)
            t.config.setConfigFilename(os.path.normpath(os.path.join(destinationDir, os.path.split(t.config.configFilename())[1])))
        self.sigStatusChanged.emit()

    def __del__(self):
        if __debug__: logging.debug('Deleted: ' + str(self))

    @property
    def location(self):
        """The directory in which the project is located.

        Example::

            print(project.location)
            '/path/to/the/directory/containing/this/project'

        :Getter:
            (str) Returns the file path of the projects parent directory.

        .. versionadded:: 5.1
            Added **location** attribute to project.
        """
        return os.path.dirname(os.path.abspath(self.get_filename()))

    def export(self, file_path, file_type='zip'):
        """Export a chipwhisperer project.

        Saves project before exporting.

        Supported export types:
          *  zip (default)

        Returns:
            (str) Path to the exported file.

        .. versionadded:: 5.1
            Add **export** method to active project.
        """
        self.save()

        _, cwp_file = os.path.split(self.get_filename())
        name, ext = os.path.splitext(cwp_file)
        data_folder = os.path.join(self.location, '_'.join([name, 'data']))

        file_paths = list()
        file_paths.append(os.path.join(self.location, cwp_file))

        for root, directories, files in os.walk(data_folder):
            for filename in files:
                file_paths.append(os.path.join(root, filename))

        if file_type == 'zip':
            file_path = os.path.abspath(file_path)
            with zipfile.ZipFile(file_path, 'w') as zip:
                common_path = os.path.commonpath(file_paths)
                for file in file_paths:
                    relative_path = os.path.relpath(file, common_path)
                    zip.write(file, arcname=relative_path)
                    export_file_path = os.path.abspath(zip.filename)
        else:
            raise ValueError('{} not supported'.format(file_type))

        return export_file_path

    def close(self, save=True):
        """Closes the project cleanly.

        Saves by default. Then closes all claimed files.

        Args:
            save (bool): Saves the project before closing.
        """
        if save:
            self.save()

        for seg in self.segments:
            seg.unloadAllTraces()

    def remove(self, i_am_sure=False):
        """Remove a project from disk.

        Args:
            i_am_sure (bool): Are you sure you want to remove the project?
        """
        if not i_am_sure:
            raise RuntimeWarning('Project not removed... i_am_sure not set to True.')

        self.close(save=False)

        try:
            shutil.rmtree(self.datadirectory)
        except FileNotFoundError:
            pass

        _, cwp_file = os.path.split(self.get_filename())
        try:
            os.remove(os.path.join(self.location, cwp_file))
        except FileNotFoundError:
            pass

    @property
    def traces(self):
        """The interface to all traces contained in the project.

        Instance of :class:`.Traces`.
        """
        return self._traces

    @property
    def segments(self):
        """The interface to all segments contained in the project.

        Instance of :class:`.Segments`.
        """
        return self._segments

    @property
    def keys(self):
        """Iterable for working with only the known keys.

        Each item in the iterable is a byte array.

        Supports iterating, indexing, and slicing::

            for key in my_project.keys:
                # do something
        """
        return self._keys

    @property
    def textins(self):
        """Iterable for working with only the text in.

        Each item in the iterable is a byte array.

        Supports iterating, indexing, and slicing::

            for textin in my_project.textins:
                # do something
        """
        return self._textins

    @property
    def textouts(self):
        """Iterable for working with only the text out.

        Each item in the iterable is a byte array.

        Supports iterating, indexing, and slicing::

            for textout in my_project.textouts:
                # do something
        """
        return self._textouts

    @property
    def waves(self):
        """Iterable for working with only the trace data.

        Each item in the iterable is a numpy array.

        Supports iterating, indexing, and slicing::

            for wave in my_project.waves:
                # do something
        """
        return self._waves


class Traces:
    """Contains the trace interface for the project.

    The class adds support for indexing, slicing and iterating
    of project traces. To add traces to the project use
    :meth:`append <.Traces.append>`. ::

        for trace in my_traces:
            my_project.traces.append(trace)

    To iterate through all traces use::

        for trace in my_project.traces:
            print(trace.wave, trace.textin, trace.textout, trace.key)

    Indexing is supported::

        trace_of_interest = my_project.traces[99]

    So is slicing::

        interesting_traces = my_project.traces[20:35]

    Args:
        project: The project class where traces will be stored.
        segment_length (int): The number of traces stored in one file
            before another file is created. This is used for unloading
            and loading traces from disk to keep memory usage reasonable.

    Returns:
        (iterable) of :class:`traces <chipwhisperer.common.traces.Trace>`
        in the project if the traces belongs
        to a segment that is enabled. The iterable contains tuples
        of length four containing the trace, plain text in,
        encrypted text out, and known key.

    Raises:
        IndexError: When the index used for indexing is out of range.
        TypeError: When key used to get item is not a slice or integer.

    .. versionadded:: 5.1
        Added **Traces** class to project interface.
    """

    def __init__(self, project, segment_length=10000):
        self.project = project
        self.tm = project._traceManager

        # segment management
        self.cur_seg = self.project.segments.new()
        self.project.segments.append(self.cur_seg)
        self.cur_trace_num = 0
        self.seg_len = segment_length
        self.seg_ind_max = self.seg_len - 1

    @property
    def max(self):
        """Max index during iteration."""
        return self.tm.num_traces() - 1
    
    @property
    def keys(self):
        return self._keys

    def append(self, trace):
        """Append a Trace containing the trace and related operation information.

        Args:
            trace (:class:`Trace <chipwhisperer.common.trace.Trace>`): A captured or created trace.

        Raises:
            TypeError: When trying to append something other than a trace.
        """
        if not isinstance(trace, Trace):
            raise TypeError("Expected Trace object, got {}.".format(trace))

        if self.cur_trace_num > self.seg_ind_max:
            self.cur_seg = self.project.segments.new()
            self.project.segments.append(self.cur_seg)
            self.cur_trace_num = 0
        self.cur_seg.add_trace(*trace)
        self.cur_trace_num += 1

    def extend(self, iterable):
        """Add all traces in an iterable to the project.

        Args:
            iterable: Any iterable of :class:`Trace <chipwhisperer.common.trace.Trace>` objects.

        Raises:
            TypeError: If any of the object in the iterable are not a trace.
        """
        for item in iterable:
            self.append(item)

    def __len__(self):
        return self.tm.num_traces()

    def __iter__(self):
        self.n = 0
        return self

    def __next__(self):
        if self.n > self.max:
            raise StopIteration

        result = Trace(
            self.tm.get_trace(self.n),
            self.tm.get_textin(self.n),
            self.tm.get_textout(self.n),
            self.tm.get_known_key(self.n),
        )

        self.n += 1
        return result

    def __getitem__(self, item):
        if isinstance(item, int):
            ind = item
            if ind < 0:
                ind = self.max + ind + 1

            if not (0 <= ind <= self.max):
                raise IndexError('Index outside of range ({}, {})'.format(0, self.max))

            result = Trace(
                self.tm.get_trace(ind),
                self.tm.get_textin(ind),
                self.tm.get_textout(ind),
                self.tm.get_known_key(ind)
            )
            return result

        elif isinstance(item, slice):
            indices = item.indices(self.tm.num_traces())
            result = []
            for i in range(*indices):
                result.append(
                    Trace(
                        self.tm.get_trace(i),
                        self.tm.get_textin(i),
                        self.tm.get_textout(i),
                        self.tm.get_known_key(i)
                    )
                )
            return result
        else:
            raise TypeError('Indexing by integer or slice only')

    def __repr__(self):
        _, project_filename = os.path.split(self.project.get_filename())
        abs_path = os.path.join(self.project.location, project_filename)
        return 'Traces(number={}, project={}) for project at {}'.format(len(self), self.project, abs_path)

    def __str__(self):
        result = []
        i = 0
        while i < 10:
            try:
                result.append(self[i])
            except IndexError:
                break
            i += 1
        return '{} \n Result truncated at maximum 10 lines...'.format(str(np.array(result)))


class Segments:
    """Contains the segment interface for a project.

    Segments are used to group traces together. Each segment can be
    enabled/disabled to be included in the traces returned by
    :attr:`.traces`

    .. warning:: For internal use only. This api is not guaranteed to exist
        if our project switches to a different way of storing project
        files.

    Returns:
        (iterable) of segments, usually some sort of trace container.

    """

    def __init__(self, project):
        self.project = project
        self.tm = project._traceManager
        self.trace_container = project._trace_format
        self.data_directory = project.datadirectory

    def __len__(self):
        return len(self.tm.traceSegments)

    def new(self):
        """Used to get a new empty trace container (segment).
        
        Returns:
            (TraceContainer) A new empty instance of a trace container.
        """
        seg = copy.copy(self.trace_container)
        seg.clear()
        start_time = datetime.now()
        prefix = start_time.strftime('%Y.%m.%d-%H.%M.%S') + "_"
        seg.config.setConfigFilename(self.data_directory + "traces/config_" + prefix + ".cfg")
        seg.config.setAttr("prefix", prefix)
        seg.config.setAttr("date", start_time.strftime('%Y-%m-%d %H:%M:%S'))
        return seg

    def append(self, seg):
        self.tm.appendSegment(seg)

    def __iter__(self):
        self.n = 0
        self.max = len(self) - 1
        return self

    def __next__(self):
        if self.n > self.max:
            raise StopIteration
        result = self.tm.traceSegments[self.n]
        self.n += 1
        return result

    def __getitem__(self, item):
        return self.tm.traceSegments[item]

    def __repr__(self):
        _, project_filename = os.path.split(self.project.get_filename())
        abs_path = os.path.join(self.project.location, project_filename)
        return 'Segments(num={}, project={}) for project at {}'.format(len(self), self.project, abs_path)


class IndividualIterable:
    
    def __init__(self, getter_func, trace_num_func):
        self.getter = getter_func
        self.trace_num_func = trace_num_func

    @property
    def max(self):
        return self.trace_num_func() - 1

    def __iter__(self):
        self.n = 0
        return self

    def __next__(self):
        if self.n > self.max:
            raise StopIteration

        self.n += 1
        return self.getter(self.n)

    def __getitem__(self, item):
        if isinstance(item, int):
            ind = item
            if ind < 0:
                ind = self.max + ind + 1

            if not (0 <= ind <= self.max):
                raise IndexError('Index outside of range ({}, {})'.format(0, self.max))

            return self.getter(ind)

        elif isinstance(item, slice):
            indices = item.indices(self.max+1)
            result = []
            for i in range(*indices):
                result.append(self.getter(i))
            return result
        else:
            raise TypeError('Indexing by integer or slice only')




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import collections
import logging

class SimpleSerialTemplate:

    """
    SimpleSerial serial reader base class.

    All SimpleSerial readers have two data buffers:
    1. A buffer for received data to go to the SimpleSerial target module. This buffer contains bytes.
    2. A buffer for sent and received data to go to the serial terminal. This buffer contains ['in'/'out', char] pairs.
    These two structures are queues with a fixed maximum size: when they overflow, the oldest data must be removed
    from the queues.

    Note that child classes should only need to implement the following:
    - hw_read()
    - hw_write()
    - hw_in_waiting()
    """

    _name= 'Simple Serial Reader'

    def __init__(self):
        self.connectStatus = False

        self.target_queue = collections.deque()
        self.target_count = 0

        self.terminal_queue = collections.deque()
        self.terminal_count = 0

        self.max_queue_size = 384

    def selectionChanged(self):
        pass

    def close(self):
        pass

    def con(self, scope=None):
        """Connect to target"""
        self.connectStatus = True

    def dis(self):
        """Disconnect from target"""
        self.close()
        self.connectStatus = False

    def flushInput(self):
        self.flush()

    def write(self, string):
        """
        Write a string to the device.

        This involves 2 actions:
        1. Write the string to the hardware;
        2. Insert ['out', b] pairs into the terminal queue

        This should be called by a target module (like SimpleSerial)

        Args:
            string: The string to be written to the device
        Returns:
            None
        """

        # Write to hardware
        self.hardware_write(string)

        # Update terminal buffer
        for c in string:
            self.terminal_queue.append(['out', c])
            if self.terminal_count < self.max_queue_size:
                self.terminal_count += 1
            else:
                self.terminal_queue.popleft()

    def read(self, num=0, timeout=250):
        """
        Attempt to read a string from the device.

        This involves three steps:
        1. Remove existing data from the target buffer
        2. If needed, request more data from the hardware
        3. If any, add received data to terminal queue in ['in', b] pairs

        This is the interface for target modules - it places the received bytes in a terminal buffer.

        Args:
            num: The number of bytes to be read. If 0, read no data.
            timeout: How long to wait before returning, in ms. If 0, block until data received.

        Returns:
            String of received data (possibly shorter than num characters)
        """

        # Try to read from queue
        ret = ''
        while num > 0 and self.target_count > 0:
            ret += self.target_queue.popleft()
            self.target_count -= 1
            num -= 1

        if num == 0:
            return ret

        # If we didn't get enough data, try to read more from the hardware
        data = bytearray(self.hardware_read(num, timeout=timeout)).decode('latin-1')
        for c in data:
            self.terminal_queue.append(['in', c])
            if self.terminal_count < self.max_queue_size:
                self.terminal_count += 1
            else:
                self.terminal_queue.popleft()
        ret += data
        return ret

    def flush(self):
        """
        Remove any waiting data from the target buffer.

        Returns:
            None
        """
        waiting = self.hardware_inWaiting()
        while waiting > 0:
            self.hardware_read(waiting)
            waiting = self.hardware_inWaiting()
        self.target_queue.clear()
        self.target_count = 0

    def inWaiting(self):
        """
        Check the number of bytes to be read (for target modules).

        This is the total amount of data between the target buffer and the hardware's buffer.

        Returns:
            int: the number of bytes waiting to be read
        """
        bbuf = self.target_count
        if bbuf == self.max_queue_size:
            logging.warning('Python SimpleSerial reader buffer OVERRUN - data loss has occurred.')
        return self.hardware_inWaiting() + bbuf

    def terminal_write(self, string):
        """
        Write a string to the device.

        No other processing is done here. This is intended for the ChipWhisperer terminal - it doesn't interact with
        the data buffers.

        Args:
            string: The string to be written

        Returns:
            None
        """
        # Write to hardware
        self.hardware_write(string)

    def terminal_read(self, num=0, timeout=250):
        """
        Attempt to read a bytestring from the device.

        This involves three steps:
        1. Remove existing data from the terminal buffer
        2. If needed, request more data from the hardware
        3. If any, add received data to target queue
        This is the interface for the ChipWhisperer terminal - it places the received bytes in a target buffer.

        Args:
            num: The number of bytes to be read. If 0, read all data available.
            timeout: How long to wait before returning, in ms. If 0, block until data received.

        Returns:
            A list of ['in'/'out', char] pairs
        """

        # Try to read from queue
        ret = []
        while num > 0 and self.terminal_count > 0:
            ret.append(self.terminal_queue.popleft())
            self.terminal_count -= 1
            num -= 1

        if num == 0:
            return ret

        # If we didn't get enough data, try to read more from the hardware
        data = self.hardware_read(num, timeout=timeout)
        data = bytearray(data)
        data = str(data)
        for c in data:
            self.target_queue.append(str(c))
            if self.target_count < self.max_queue_size:
                self.target_count += 1
            else:
                self.target_queue.popleft()
            ret.append(['in', c])
        return ret

    def terminal_flush(self):
        """
        Remove all data waiting for the terminal. Don't remove data currently waiting in the hardware.

        Returns:
            None
        """

        self.terminal_queue.clear()
        self.terminal_count = 0

    def terminal_inWaiting(self):
        """
        Check the number of bytes to be read (for terminal).

        This is the total amount of data between the terminal buffer and the hardware's buffer.

        Returns:
            int: the number of bytes waiting to be read
        """

        bbuf = self.terminal_count
        if bbuf == self.max_queue_size:
            logging.warning('Python SimpleSerial reader buffer OVERRUN - data loss has occurred.')
        return self.hardware_inWaiting() + bbuf

    def hardware_inWaiting(self):
        """
        Check how many bytes are in waiting on the device's hardware buffer.

        This function needs to be implemented in child classes.

        Returns:
            int: number of bytes waiting to be read
        """
        raise NotImplementedError

    def hardware_write(self, string):
        """
        Write a string to the target.

        This function needs to be implemented in child classes.

        Returns:
            None
        """
        raise NotImplementedError

    def hardware_read(self, num, timeout=250):
        """
        Read a number of bytes from the hardware.

        This function needs to be implemented in child classes.

        Args:
            num: The number of bytes to be read. If 0, read all data available.
            timeout: How long to wait before returning, in ms. If 0, block until data received.

        Returns:
            String of received data (possibly shorter than num characters)
        """
        raise NotImplementedError



#!/usr/bin/python
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_WARN
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2018, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.github.com/newaetech/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import numpy as np
from usb import USBError
from .base import ScopeTemplate
from chipwhisperer.capture.scopes.openadc_interface.naeusbchip import OpenADCInterface_NAEUSBChip
from chipwhisperer.common.utils import util, timer
from chipwhisperer.common.utils.util import dict_to_str
from collections import OrderedDict

from chipwhisperer.hardware.naeusb.serial import USART
from chipwhisperer.hardware.naeusb.naeusb import NAEUSB, packuint32, unpackuint32
from chipwhisperer.hardware.naeusb.programmer_avr import AVRISP
from chipwhisperer.hardware.naeusb.programmer_xmega import XMEGAPDI
from chipwhisperer.hardware.naeusb.programmer_stm32fserial import STM32FSerial
from chipwhisperer.common.utils.util import camel_case_deprecated
import time
import datetime



class ADCSettings(util.DisableNewAttr):
    USB_ADCLK_SET = 0x28
    USB_SAMPLES = 0x2A


    def __init__(self, usb):
        self.usb = usb
        self.disable_newattr()

    def _dict_repr(self):
        dict = OrderedDict()
        dict['clk_src'] = self.clk_src
        dict['clk_freq'] = self.clk_freq
        dict['samples'] = self.samples
        return dict

    def __repr__(self):
        return util.dict_to_str(self._dict_repr())

    def __str__(self):
        return self.__repr__()

    @property
    def samples(self):
        """Number of samples to store."""

        resp = self.usb.readCtrl(self.USB_SAMPLES, 0, 4)
        return unpackuint32(resp)

    @samples.setter
    def samples(self, numsamples):

        nresp = packuint32(numsamples)
        self.usb.sendCtrl(self.USB_SAMPLES, 0, nresp)


    @property
    def clk_src(self):
        """ADC Clock source.

        :Getter: Returns 'int' or 'ext' based on the clock source.

        :Setter: (str) Set the ADC clock source to either internal or external:
            ('int' or 'ext')
        """

        resp = self.usb.readCtrl(self.USB_ADCLK_SET, 0, 5)
        if resp[3] == 0:
            return 'int'
        else:
            return 'ext'


    @clk_src.setter
    def clk_src(self, src):

        if src == "int":
            src = 0
        elif src == "ext":
            src = 1
        else:
            raise ValueError("Invalid source: %s"%str(src))

        resp = self.usb.readCtrl(self.USB_ADCLK_SET, 0, 5)

        resp[3] = src
        resp[4] = 1

        self.usb.sendCtrl(self.USB_ADCLK_SET, 0, 5)

    @property
    def clk_freq(self):
        """Set the frequency for CLKOUT. Will be rounded to nearest possible values, check results to see
        programmed value. Set to 'None' for disabling (High-Z) output.

        :Getter: Returns the actual frequency for CLKOUT

        :Setter: Sets CLKOUT to the nearest possible value.
        """

        resp = self.usb.readCtrl(self.USB_ADCLK_SET, 0, 5)
        if resp[0] == 0:
            return 0
        else:
            return 240E6/resp[0]

    @clk_freq.setter
    def clk_freq(self, freqset):

        resp = self.usb.readCtrl(self.USB_ADCLK_SET, 0, 5)

        if freqset is None:
            best_div = 0
        else:
            #Get as close as possible - 240 MHz clock
            #Divider options: 1,2,4,8,16,32,64
            div_options = [1,2,4,8,16,32,64]

            freqsrc = 240E6

            err_list = [100E6]*len(div_options)

            for i, div in enumerate(div_options):
                err_list[i] = abs((freqsrc / div) - freqset)

            best_div = div_options[np.argmin(err_list)]

        resp[0] = best_div
        resp[4] = 1

        self.usb.sendCtrl(self.USB_ADCLK_SET, 0, resp)

        if best_div == 0:
            return 0
        else:
            return 240E6 / best_div

class GlitchSettings(util.DisableNewAttr):
    USB_GLITCH_SETTINGS = 0x2C
    USB_GLITCH_GO = 0x2D

    def __init__(self, usb):
        self.usb = usb
        self.disable_newattr()


    def _dict_repr(self):
        dict = OrderedDict()
        dict['repeat'] = self.repeat
        dict['ext_offset'] = self.ext_offset
        return dict

    def __repr__(self):
        return util.dict_to_str(self._dict_repr())

    def __str__(self):
        return self.__repr__()

    @property
    def ext_offset(self):
        """Offset from rising edge of trigger & when glitch gets inserted, approx = 8.3 ns * ext_offset"""
        resp = self.usb.readCtrl(self.USB_GLITCH_SETTINGS, 0, 8)[0:4]
        return unpackuint32(resp)

    @ext_offset.setter
    def ext_offset(self, offset):

        resp = self.usb.readCtrl(self.USB_GLITCH_SETTINGS, 0, 8)
        nresp = packuint32(offset)
        nresp.extend(resp[4:8])
        self.usb.sendCtrl(self.USB_GLITCH_SETTINGS, 0, nresp)

    @property
    def repeat(self):
        """Width of glitch in cycles (approx = 8.3 ns * width)"""

        resp = self.usb.readCtrl(self.USB_GLITCH_SETTINGS, 0, 8)[4:8]
        return unpackuint32(resp)

    @repeat.setter
    def repeat(self, width):
        resp = self.usb.readCtrl(self.USB_GLITCH_SETTINGS, 0, 8)
        nresp = resp[0:4]
        nresp.extend(packuint32(width))
        self.usb.sendCtrl(self.USB_GLITCH_SETTINGS, 0, nresp)

    def manualTrigger(self):
        """Manually trigger the glitch"""
        self.insert()

    def insert(self):
        self.usb.sendCtrl(self.USB_GLITCH_GO)


class GPIOSettings(util.DisableNewAttr):


    USB_MODE_OUT = 0x01
    USB_MODE_IN = 0x02
    USB_MODE_LOW = 0x03
    USB_MODE_HIGH = 0x04
    USB_MODE_PERA = 0x05
    USB_MODE_PERB = 0x06

    USB_GPIO_READDIR = 0x24
    USB_GPIO_SET   = 0x25
    USB_GPIO_READ  = 0x26

    USB_CLKOUT_SET = 0x27

    USB_GPIO1_MASK = (1<<0)
    USB_GPIO2_MASK = (1<<1)
    USB_GPIO3_MASK = (1<<2)
    USB_GPIO4_MASK = (1<<3)
    USB_PDIC_MASK = (1<<4)
    USB_PDID_MASK = (1<<5)
    USB_nRST_MASK = (1<<6)

    def __init__(self, usb):
        self.usb = usb
        self.disable_newattr()


    def _dict_repr(self):
        dict = OrderedDict()
        dict['tio1'] = self.tio1
        dict['tio2'] = self.tio2
        dict['tio3'] = self.tio3
        dict['tio4'] = self.tio4

        dict['pdid'] = self.pdid
        dict['pdic'] = self.pdic
        dict['nrst'] = self.nrst

        #dict['glitch_lp'] = self.glitch_lp

        dict['clkout'] = self.clkout

        return dict

    def __repr__(self):
        return util.dict_to_str(self._dict_repr())

    def __str__(self):
        return self.__repr__()

    @property
    def tio1(self):
        """The function of the Target IO1 pin.

        TIO1 can be used for the following functions:
          * "serial_rx": UART input
          * "high_z" / None: High impedance input
          * "gpio_low" / False: Driven output: logic 0
          * "gpio_high" / True: Driven output: logic 1
          * "gpio_disabled": Driven output: no effect

        Default value is "serial_rx".

        :Getter: Return one of the above strings

        :Setter: Set the Target IO1 mode.
            Raises: ValueError if new value is not one of the above modes
        """
        return None

    @tio1.setter
    def tio1(self, state):
        state = self._gpio_name_to_state(state)
        if state == "serial_rx":
            state = "pera"

        self.gpio_generic_set(self.USB_GPIO1_MASK, state)

    @property
    def tio2(self):
        """The function of the Target IO2 pin.

        TIO2 can be used for the following functions:
          * "serial_tx": UART output
          * "high_z" / None: High impedance input
          * "gpio_low" / False: Driven output: logic 0
          * "gpio_high" / True: Driven output: logic 1
          * "gpio_disabled": Driven output: no effect

        Default value is "serial_tx".

        :Getter: Return one of the above strings

        :Setter: Set the Target IO2 mode.
            Raises: ValueError if new value is not one of the above modes
        """
        return None

    @tio2.setter
    def tio2(self, state):
        state = self._gpio_name_to_state(state)
        if state == "serial_tx":
            state = "pera"

        self.gpio_generic_set(self.USB_GPIO2_MASK, state)

    @property
    def tio3(self):
        """The function of the Target IO3 pin.

        TIO3 can be used for the following functions:
          * "high_z" / None: High impedance input
          * "gpio_low" / False: Driven output: logic 0
          * "gpio_high" / True: Driven output: logic 1
          * "gpio_disabled": Driven output: no effect

        Default value is "high_z".

        :Getter: Return one of the above strings

        :Setter: Set the Target IO3 mode.
            Raises: ValueError if new value is not one of the above modes
        """
        return None

    @tio3.setter
    def tio3(self, state):
        state = self._gpio_name_to_state(state)
        self.gpio_generic_set(self.USB_GPIO3_MASK, state)

    @property
    def tio4(self):
        """The function of the Target IO4 pin.

        TIO4 can be used for the following functions:
          * "high_z" / None: High impedance input
          * "gpio_low" / False: Driven output: logic 0
          * "gpio_high" / True: Driven output: logic 1
          * "gpio_disabled": Driven output: no effect

        Default value is "high_z". Typically, this pin is used as a trigger
        input.

        :Getter: Return one of the above strings

        :Setter: Set the Target IO4 mode
            Raises: ValueError if new value is not one of the above modes
        """
        return None

    @tio4.setter
    def tio4(self, state):
        state = self._gpio_name_to_state(state)
        self.gpio_generic_set(self.USB_GPIO4_MASK, state)

    @property
    def pdic(self):
        """The state of the PDIC pin output pin.

        This is a GPIO pin. The following values are allowed:
          * "high" / True: logic 1
          * "low" / False: logic 0
          * "disabled" / "default" / "high_z" / None: undriven

        :Getter: Return one of "high", "low", or "high_z"

        :Setter: Set the pin's state
            Raises: ValueError if new state not listed above
        """
        return False

    @pdic.setter
    def pdic(self, state):
        state = self._gpio_name_to_state(state)
        self.gpio_generic_set(self.USB_PDIC_MASK, state)

    @property
    def pdid(self):
        """The state of the PDID pin.

        See pdic for more information."""
        return True

    @pdid.setter
    def pdid(self, state):
        state = self._gpio_name_to_state(state)
        self.gpio_generic_set(self.USB_PDID_MASK, state)

    @property
    def nrst(self):
        """The state of the NRST pin.

        See pdic for more information."""
        return True

    @nrst.setter
    def nrst(self, state):
        state = self._gpio_name_to_state(state)
        self.gpio_generic_set(self.USB_nRST_MASK, state)


    @property
    def clkout(self):
        """"Set the frequency for CLKOUT. Will be rounded to nearest possible values, check results to see
        programmed value. Set to 'None' for disabling (High-Z) output."""

        resp = self.usb.readCtrl(self.USB_CLKOUT_SET, 0, 3)
        if resp[0] == 0:
            return 0
        else:
            return 240E6/resp[0]

    @clkout.setter
    def clkout(self, freqset):

        if freqset is None:
            best_div = 0
        else:
            #Get as close as possible - 240 MHz clock
            #Divider options: 1,2,4,8,16,32,64
            div_options = [1,2,4,8,16,32,64]

            freqsrc = 240E6

            err_list = [100E6]*len(div_options)

            for i, div in enumerate(div_options):
                err_list[i] = abs((freqsrc / div) - freqset)

            best_div = div_options[np.argmin(err_list)]

        self.usb.sendCtrl(self.USB_CLKOUT_SET, 0, [best_div, 0, 0])

        if best_div == 0:
            return 0
        else:
            return 240E6 / best_div


    def gpio_generic_set(self, gpiomask, state, setdir=True):
        if state is None:
            self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_IN, [gpiomask, 0])

        elif isinstance(state, int):
            #If binary/int, check true/false
            if state:
                if setdir:
                    self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_OUT, [gpiomask, 0])
                self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_HIGH, [gpiomask, 0])
            else:
                if setdir:
                    self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_OUT, [gpiomask, 0])
                self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_LOW, [gpiomask, 0])

        elif state == "pera":
            self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_PERA, [gpiomask, 0])

        elif state == "perb":
            self.usb.sendCtrl(self.USB_GPIO_SET, self.USB_MODE_PERB, [gpiomask, 0])

        else:
            raise ValueError("Unknown pin state: %s"%state)

    def _gpio_state_to_name(self, state):
        if state is None:
            return "high_z"
        elif state:
            return "high"
        else:
            return "low"

    def _gpio_name_to_state(self, state):
        if (state == "gpio_low") or (state == "low"):
            state = False
        elif (state == "gpio_high") or (state == "high"):
            state = True
        elif (state == "high_z") or (state == "default") or (state == "disabled"):
            state = None

        return state


class CWNano(ScopeTemplate, util.DisableNewAttr):
    """CWNano scope object.

    This class contains the public API for the CWNano hardware. It includes
    specific settings for each of these devices.

    To connect to one of these devices, the easiest method is::

        import chipwhisperer as cw
        scope = cw.scope(type=scopes.CWNano)

    Some sane default settings can be set using::

        scope.default_setup()

    For more help about scope settings, try help() on each of the ChipWhisperer
    scope submodules (scope.adc, scope.io, scope.glitch):

      * :attr:`scope.adc <.CWNano.adc>`
      * :attr:`scope.io <.CWNano.io>`
      * :attr:`scope.glitch <.CWNano.glitch>`
      * :meth:`scope.default_setup <.CWNano.default_setup>`
      * :meth:`scope.con <.CWNano.con>`
      * :meth:`scope.dis <.CWNano.dis>`
      * :meth:`scope.get_last_trace <.CWNano.get_last_trace>`
      * :meth:`scope.arm <.CWNano.arm>`
      * :meth:`scope.capture <.CWNano.capture>`
    """

    _name = "ChipWhisperer Nano"

    REQ_ARM = 0x29
    REQ_SAMPLES = 0x2A

    def __init__(self):
        ScopeTemplate.__init__(self)
        self._is_connected = False


        self._cwusb = NAEUSB()
        self.ser = self._cwusb
        self.scopetype = self
        self.dev = self

        self.xmega = XMEGAPDI(self._cwusb)
        self.avr = AVRISP(self._cwusb)
        self.usart = USART(self._cwusb)
        self.serialstm32f = STM32FSerial(cwserial=self.usart)
        self.serialstm32f.scope = self
        self.io = GPIOSettings(self._cwusb)
        self.adc = ADCSettings(self._cwusb)
        self.glitch = GlitchSettings(self._cwusb)
        self._timeout = 2

        self._lasttrace = None

        self.disable_newattr()

    def default_setup(self):
        """ Sets up sane capture defaults for this scope

          * 7.5MHz ADC clock
          * 7.5MHz output clock
          * 5000 capture samples
          * tio1 = serial rx
          * tio2 = serial tx
          * glitch module off

        .. versionadded:: 5.1
            Added default setup for CWNano
        """
        self.adc.clk_freq = 7.5E6
        self.io.clkout = 7.5E6
        self.adc.samples = 5000
        self.io.tio1 = "serial_rx"
        self.io.tio2 = "serial_tx"
        self.glitch.repeat = 0

    def getCurrentScope(self):
        return self

    def _getNAEUSB(self):
        return self._cwusb

    def _con(self, sn=None):
        try:
            possible_sn = self._cwusb.get_possible_devices(idProduct=[0xACE0])
            serial_numbers = []
            if len(possible_sn) > 1:
                if sn is None:
                    for d in possible_sn:
                        serial_numbers.append("sn = {} ({})".format(str(d['sn']), str(d['product'])))
                    raise Warning("Multiple ChipWhisperers detected. Please specify device from the following list using cw.scope(sn=<SN>): \n{}".format(serial_numbers))
            else:
                sn = None
            found_id = self._cwusb.con(idProduct=[0xACE0], serial_number=sn)
        except (IOError, ValueError):
            raise Warning("Could not connect to cwnano. It may have been disconnected, is in an error state, or is being used by another tool.")
        self.disable_newattr()
        self._is_connected = True
        return True

    def _dis(self):
        self.enable_newattr()
        self._is_connected = False
        return True

    def arm(self):
        """Arm the ADC, the trigger will be GPIO4 rising edge (fixed trigger)."""
        if self.connectStatus is False:
            raise Warning("Scope \"" + self.getName() + "\" is not connected. Connect it first...")

        self._cwusb.sendCtrl(self.REQ_ARM, 1)


    def capture(self):
        """Raises IOError if unknown failure, returns 'True' if timeout, 'False' if no timeout"""

        starttime = datetime.datetime.now()
        while self._cwusb.readCtrl(self.REQ_ARM, dlen=1)[0] == 0:
            # Wait for a moment before re-running the loop
            time.sleep(0.05)
            diff = datetime.datetime.now() - starttime

            # If we've timed out, don't wait any longer for a trigger
            if (diff.total_seconds() > self._timeout):
                logging.warning('Timeout in cwnano capture()')
                return True

        self._lasttrace = self._cwusb.cmdReadMem(0, self.adc.samples)

        self._lasttrace = np.array(self._lasttrace) / 256.0 - 0.5

        #self.newDataReceived(0, self._lasttrace, 0, self.adc.clk_freq)

        return False


    def get_last_trace(self):
        """Return the last trace captured with this scope.
        """
        return self._lasttrace

    getLastTrace = camel_case_deprecated(get_last_trace)


    def _dict_repr(self):
        dict = OrderedDict()
        dict['io']    = self.io._dict_repr()
        dict['adc']   = self.adc._dict_repr()
        dict['glitch'] = self.glitch._dict_repr()
        return dict

    def __repr__(self):
        # Add some extra information about ChipWhisperer type here
        if self._is_connected:
            ret = "ChipWhisperer Nano Device\n"
            return ret + dict_to_str(self._dict_repr())
        else:
            ret = "ChipWhisperer Nano device (disconnected)"
            return ret

    def __str__(self):
        return self.__repr__()

    def get_possible_devices(self, idProduct):
        return self._cwusb.get_possible_devices(idProduct=idProduct)

    def usbdev(self):
        return self._cwusb



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from ._base import PreprocessingBase
import scipy
import scipy.signal
import scipy.fftpack
import numpy as np
from matplotlib.mlab import find


def fft(signal, freq=None):
    FFT = abs(scipy.fft(signal))
    # FFTdb = 20*scipy.log10(FFT)
    # freqs = scipy.fftpack.fftfreq(len(signal), 1/freq)

    # FFTdb = FFTdb[2:len(freqs)/2]
    # freqs = freqs[2:len(freqs)/2]
    # return (freqs, FFTdb)

    print(FFT)


class DecimationClockRecovery(PreprocessingBase):
    """
    Attempts Clock recovery & then decimates based on that. Currently broken.
    """
    _name = "Decimation: Clock Recovery"
    _description = "Attempts to 'recover' the clock by band-pass filtering, and then uses that to "\
                  "decimate to only points of interest.\n****CURRENTLY NOT SUPPORTED****"

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self.setFilterOptions()
        self.getParams().addChildren([
            {'name':'Filter Design', 'type':'group', 'children':[
                # {'name':'Form', 'key':'form', 'type':'list', 'values':{"Butterworth":sp.signal.butter}, 'set':self.updateScript},
                {'name':'Type', 'key':'type', 'type':'list', 'values':["bandpass"], 'default':'bandpass', 'value':'bandpass'},
                {'name':'Critical Freq BW (%)', 'key':'freqbw', 'type':'float', 'limits':(0, 200), 'step':1, 'default':20, 'value':20},
                {'name':'Recalc Passband/Trace', 'key':'recalcpertrace', 'type':'bool', 'default':False, 'value':False},
                {'name':'Order', 'key':'order', 'type':'int', 'limits':(1, 32), 'default':3, 'value':3},
             ]},
            {'name':'Enable Zero-Crossing', 'key':'enableZero', 'type':'bool', 'default':True, 'value':True},
            {'name':'Enable Decimation by ZC', 'key':'decimate', 'type':'bool', 'default':True, 'value':True},
        ])
   
    def setFilterOptions(self, recalcPerTrace=False, enableZC=True, enableDecimation=True):
        self._recalcPerTrace = recalcPerTrace
        self._enableZeroCrossing = enableZC
        self._enableDecimation = enableDecimation
   
    def setFilterParams(self, form='low', freqbw=0.15, order=5, tnum=0, useCached=False):
        if useCached:
            freqbw = self._freqbw
            form = self._form
            order = self._order
        else:
            self._freqbw = freqbw
            self._form = form
            self._order = order

        try:
            fftdata = scipy.fft(self._traceSource.getTrace(tnum))
            fftdata = abs(fftdata[2:(len(fftdata) / 2)])
            maxindx = fftdata.argmax() + 2
            centerfreq = float(maxindx) / float(len(fftdata) + 2)
            # centerfreq in range 0 - 1 now

        except:
#            print "FFT FAILED!!!"
            self.b = None
            self.a = None
            return

        freq = ((centerfreq - (centerfreq * freqbw) / 2), (centerfreq + (centerfreq * freqbw) / 2))

        logging.info('Designing filter for passband: %f-%f' % (freq[0], freq[1]))

        b, a = scipy.signal.butter(order, freq, form)
        self.b = b
        self.a = a
   
    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None
            
            if self.a is None:
                self.setFilterParams(useCached=True)
                
            if self._recalcPerTrace:
                self.setFilterParams(tnum=n, useCached=True)

            # Filter Trace
            inputtrace = scipy.signal.lfilter(self.b, self.a, trace)
            
            if self._enableZeroCrossing:

                # Problem: the filter has some start-up time, so we can't use data right away. But
                #         if you only api a waveform AFTER the trigger, this means you need to
                #         throw away data from the api until the filter is running. If the system
                #         clock frequency changes, you've now thrown away a differing amount of clock
                #         cycles and syncronization is lost.
                #
                # For now we just throw away a fixed number of samples, but it doesn't work for varying
                # clock frequencies.
                for i in range(0, 250):
                    inputtrace[i] = 0

                indices = find((inputtrace[1:] >= 0) & (inputtrace[:-1] < 0))
                filttrace = np.zeros(len(inputtrace))

                if self._enableDecimation:
                    for idx, val in enumerate(indices):
                        filttrace[idx] = trace[val]
                else:
                    for i in indices:
                        filttrace[i] = 1
            else:
                filttrace = inputtrace

            #print len(trace)
            #print len(filttrace)
            return filttrace
        else:
            return self._traceSource.getTrace(n)

    def getSampleRate(self):
        return 0  # TODO: it is not zero!



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
import os
import xml.etree.ElementTree as ET

class tracereader_dpacontestv3:
    def __init__(self):
        self.numTrace = None
        self.numPoint = None
        self.textints = None
        self.textouts = None
        self.knownkey = None
        self.directory = None
        self.tracedtype = np.int16

    def loadInfo(self, directory=None):
        if directory == None:
            directory = self.directory
        else:
            self.directory = directory
            
        tree = ET.parse(os.path.join(directory, 'info.xml'))
        root = tree.getroot()
        self.xmlroot = root
        numTrace = int(root.findall('NumTrace')[0].text)
        numPoint = int(root.findall('NumPoint')[0].text)

        self.numTrace = numTrace
        self.numPoint = numPoint
        
    def loadAllTraces(self, directory=None):
        if directory == None:
            directory = self.directory
        else:
            self.directory = directory
            
        if (self.numTrace == None):
            self.loadInfo(directory)

        self.traces = np.loadtxt(os.path.join(directory, 'wave.txt'), ndmin=2)

        try:
            self.textins = np.loadtxt(os.path.join(directory, 'text_in.txt'), dtype='|S2',  ndmin=2)
        except IOError:
            self.textins = None

        try:
            self.textouts = np.loadtxt(os.path.join(directory, 'text_out.txt'), dtype='|S2',  ndmin=2)
        except IOError:
            self.textouts = None

        try:
           knownkey = np.loadtxt(os.path.join(directory, 'key.txt'), dtype='|S2',  ndmin=2)
        except IOError:
            knownkey = None

        try:
            knownkey = knownkey[0]
            self.knownkey = []
            for r in knownkey:
                self.knownkey.append(int(r, 16))
        except:
            self.knownkey = None

    def numPoints(self):
        return self.numPoint

    def numTraces(self):
        return self.numTrace

    def getTrace(self, n):
        data = self.traces[n]

        #Following line will normalize all traces relative to each
        #other by mean & standard deviation
        #data = (data - np.mean(data)) / np.std(data)
        
        return data

    def getTextin(self, n):
        tin = self.textins[n]
        return list(map(int, tin, [16]*len(tin)))

    def getTextout(self, n):
        tout = self.textouts[n]
        return list(map(int, tout, [16]*len(tout)))

    def getKnownKey(self):
        return self.knownkey

#Test Example
if __name__ == "__main__":
    tr = tracereader_dpacontestv3()
    tr.loadAllTraces("api-2012.11.30-19.22.58")

    
    



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import uuid
from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.parameter import Parameterized, setupSetParam

class TraceSource(object):
    """
    It has traces as output
    Keeps a dictionary with all the registered objets and emits a signal when a new one is added
    """
    registeredObjects = util.DictType()
    registeredObjects["None"] = None
    sigRegisteredObjectsChanged = util.Signal()

    def __init__(self, name="Unknown"):
        self.sigTracesChanged = util.Signal()
        self.name = name
        self.uuid = str(uuid.uuid4())

    def getTrace(self, n):
        """Return the trace with number n in the current TraceSource object"""
        return None

    def numPoints(self):
        return 0

    def numTraces(self):
        return 0

    def offset(self):
        return 0

    def getSampleRate(self):
        """Return the Sample Rate used to generate the traces"""
        return 0

    def getTextin(self, n):
        """Get text-in number n"""
        raise NotImplementedError

    def getTextout(self, n):
        """Get text-out number n"""
        raise NotImplementedError

    def getKnownKey(self, n=None):
        """Get known-key number n"""
        raise NotImplementedError

    def getSegmentList(self):
        """Return a list of segments."""
        raise NotImplementedError

    def getAuxData(self, n, auxDic):
        """Return data about a segment"""
        raise NotImplementedError

    def getSegment(self, n):
        """Return the trace segment with the specified trace in the list with all enabled segments."""
        raise NotImplementedError

    def register(self):
        """Register the current TraceSource object in a list and emit a signal to inform it was updated"""
        self.registeredObjects[self.name] = self
        self.sigRegisteredObjectsChanged.emit()
        return self

    def deregister(self):
        """Deregister the current TraceSource and emit a signal to inform this event"""
        try:
            #Only deregister if UUID matches (NB: don't use direct comparison in case later we adjust repr() )
            if TraceSource.registeredObjects[self.name].uuid == self.uuid:
                if TraceSource.registeredObjects.pop(self.name, None):
                    TraceSource.sigRegisteredObjectsChanged.emit()
        except KeyError:
            pass

    @classmethod
    def deregisterObject(cls, name, uuid = None):
        """Deregister the TraceSource and emit a signal to inform this event"""
        try:
            if (uuid is None) or (cls.registeredObjects[name].uuid == uuid):
                if cls.registeredObjects.pop(name, None):
                    cls.sigRegisteredObjectsChanged.emit()
        except KeyError:
            pass

class PassiveTraceObserver(Parameterized):
    """Processes data from a TraceSource when requested """

    def __init__(self):
        self._traceSource = None
        self.getParams().addChildren([
            {'name':'Input', 'key':'input', 'type':'list', 'values':TraceSource.registeredObjects, 'default':None, 'get':self.getTraceSource, 'set':self.setTraceSource}
        ])

    @setupSetParam('Input')
    def set_trace_source(self, traceSource):
        if self._traceSource:
            self._traceSource.sigTracesChanged.disconnect(self.tracesUpdated)
        if traceSource:
            traceSource.sigTracesChanged.connect(self.tracesUpdated)
        self._traceSource = traceSource
        self.tracesUpdated()

    setTraceSource = set_trace_source

    def get_trace_source(self):
        return self._traceSource

    getTraceSource = get_trace_source

    def tracesUpdated(self):
        """Trace source was updated. Time to do something about"""
        pass

    def processTraces(self):
        """Process the Traces acording to its current state"""
        pass

    def traceSourcesChanged(self):
        """Update the Input parameter values (in the combobox).
        Usually called when TraceSource.sigRegisteredObjectsChanged emits a Signal. Should be connected manually though.
         """
        par = self.findParam('input')
        par.setLimits({})  # Will not update if the obj is the same :(
        par.setLimits(TraceSource.registeredObjects)
        if par.getValue() not in list(TraceSource.registeredObjects.values()):
            par.setValue(None)




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

import os
import numpy as np
from datetime import datetime
from ._base import TraceContainer

class TraceContainerNative(TraceContainer):
    """ Most common TraceContainer for CW Projects

    Stick to add_trace(). Trust me.
    """
    _name = "ChipWhisperer/Native"
    
    def default_config_setup(self, project):
        starttime = datetime.now()
        prefix = starttime.strftime('%Y.%m.%d-%H.%M.%S') + "_"
        self.config.setConfigFilename(os.path.join(project.datadirectory, "traces", "config_" + prefix + ".cfg"))
        self.config.setAttr("prefix", prefix)
        self.config.setAttr("date", starttime.strftime('%Y-%m-%d %H:%M:%S'))

    def copyTo(self, srcTraces=None):
        self.numTrace = srcTraces.numTraces()
        self.numPoint = srcTraces.numPoints()
        self.knownkey = srcTraces.knownkey

        self.textins = np.zeros([self.numTrace, 16], dtype=np.uint8)
        for n in range(0, self.numTrace):
            tin = srcTraces.textins[n]
            self.textins[n] = list(map(int, tin, [16] * len(tin)))

        self.textouts = np.zeros([self.numTrace, 16], dtype=np.uint8)
        for n in range(0, self.numTrace):
            tout = srcTraces.textouts[n]
            self.textouts[n] = list(map(int, tout, [16] * len(tout)))

        if srcTraces.tracedtype:
            userdtype = srcTraces.tracedtype
        else:
            userdtype = np.float

        self.traces = np.array(srcTraces.traces, dtype=userdtype)

        # Traces copied in means not saved
        self.setDirty(True)

    def loadAllTraces(self, directory=None, prefix=""):
        """Load all traces into memory"""

        if self.config.configFilename():
            if directory is None:
                directory = os.path.split(self.config.configFilename())[0]
            if prefix is None or prefix == '':
                prefix = self.config.attr("prefix")

        self.traces = np.load(os.path.join(directory, "%straces.npy" % prefix), mmap_mode='r', allow_pickle=True)
        self.textins = np.load(os.path.join(directory, "%stextin.npy" % prefix), allow_pickle=True)
        self.textouts = np.load(os.path.join(directory, "%stextout.npy" % prefix), allow_pickle=True)

        try:
            self.knownkey = np.load(os.path.join(directory, "%sknownkey.npy" % prefix), allow_pickle=True)
        except IOError:
            self.knownkey = None

        # OK if this fails
        try:
            self.keylist = np.load(os.path.join(directory, "%skeylist.npy" % prefix), allow_pickle=True)
        except IOError:
            self.keylist = None

        # Traces loaded means saved
        self.setDirty(False)
        self._isloaded = True

    def unloadAllTraces(self):
        """Drop traces from memory to save space """
        self.traces = None
        self.textins = None
        self.textouts = None
        self.knownkey = None
        self.keylist = None
        self._isloaded = False

    def saveAuxData(self, data, configDict, filenameKey="filename"):
        path = os.path.dirname(self.config.configFilename())
        prefix = self.config.attr("prefix")
        fname = "%s%s_aux_%04d.npy" % (prefix, configDict["moduleName"], configDict["auxNumber"])
        np.save(path + "/" + fname, data)

        # if filenameKey is not None:
        configDict["values"][filenameKey]["value"] = fname
        configDict["values"][filenameKey]["changed"] = True
        self.config.syncFile(sectionname=configDict["sectionName"])
        self.config.saveTrace()

        return fname

    def loadAuxData(self, fname):
        path = os.path.dirname(self.config.configFilename())
        # prefix = self.config.attr("prefix")
        # fname = "%s%s.npy" % (prefix, extraname)
        file_path = os.path.join(path, fname)
        return np.load(file_path)

    def saveAllTraces(self, directory, prefix=""):
        self.config.saveTrace()
        np.save(os.path.join(directory, "%straces.npy" % prefix), self.traces)
        np.save(os.path.join(directory, "%stextin.npy" % prefix), self.textins)
        np.save(os.path.join(directory, "%stextout.npy" % prefix), self.textouts)
        np.save(os.path.join(directory, "%skeylist.npy" % prefix), self.keylist)
        np.save(os.path.join(directory, "%sknownkey.npy" % prefix), self.knownkey)
        self.setDirty(False)

    def closeAll(self, clearTrace=True, clearText=True, clearKeys=True):
        self.saveAllTraces(os.path.dirname(self.config.configFilename()), prefix=self.config.attr("prefix"))

        # Release memory associated with data in case this isn't deleted
        if clearTrace:
            self.traces = None

        if clearText:
            self.textins = None
            self.textouts = None

        if clearKeys:
            self.keylist = None
            self.knownkey = None



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from ._base import ProtocolTemplate


class ProtocolJCardTest(ProtocolTemplate):
    _name = "JCARD"

    def loadEncryptionKey(self, key):
        pass

    def loadInput(self, inputtext):
        pass

    def go(self):

        # self.hw.scserv.connection.disconnect()
        # self.hw.con()

        status = self.hw.sendAPDU(0x00, 0xA4, 0x04, 0x00, [0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0x01, 0x01])
        if status != 0x9000:
            raise IOError("Invalid Status: %x" % status)

        (resp, pay) = self.hw.sendAPDU(0x80, 0x10, 0x01, 0x02, [0xAE], rxdatalen=6)
        if  resp != 0x9000:
            raise IOError("Invalid Status: %x" % status)

        counter1 = (pay[5] << 8) | pay[6]
        counter2 = (pay[7] << 8) | pay[8]
        counter3 = (pay[9] << 8) | pay[10]

        self.resp = (counter1, counter2, counter3)
        logging.debug(str(self.resp))

    def readOutput(self):
        return self.resp



#! /usr/bin/env python
"""\
Scan for serial ports.

Part of pySerial (http://pyserial.sf.net)
(C) 2002-2003 <cliechti@gmx.net>

The scan function of this module tries to open each port number
from 0 to 255 and it builds a list of those ports where this was
successful.
"""

import platform
import glob
import serial
list_ports = None
try:
    from serial.tools import list_ports
except ImportError:
    pass


def scan(max_range=256):
    """scan for available ports. return a list of names"""
    if not list_ports:
        # Very old pyserial compatibility mode
        system_name = platform.system()
        if system_name == 'Windows':
            available = []
            for i in range(max_range):
                try:
                    s = serial.Serial("COM%d" % i)
                    available.append(s.portstr)
                    s.close()   # explicit close 'cause of delayed GC in java
                except serial.SerialException:
                    pass
            return available
        elif system_name == 'Darwin':
            return glob.glob('/dev/cu.*')
        return glob.glob('/dev/ttyS*') + glob.glob('/dev/ttyACM*') + glob.glob('/dev/ttyUSB*')

    ports = list_ports.comports()
    # Newer pyserial returns objects with attributes, older versions
    # were returning each port as a list.
    if hasattr(ports, 'device'):
        ports = [x.device for x in ports]
    else:
        ports = [x[0] for x in ports]
    return sorted(ports)[:max_range]

if __name__=='__main__':
    print ("Found ports:")
    for n,s in enumerate(scan()):
        print(("(%d) %s" % (n,s)))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#==========================================================================

import math
import time

class PLLCDCE906(object):

    def __init__(self, usb, ref_freq):
        self._usb = usb
        self.reffreq = ref_freq
        self._pll0source = 'PLL0'
        self._pll0slew = '+0nS'
        self._pll1slew = '+0nS'
        self._pll2slew = '+0nS'

    def pll_outfreq_set(self, freq, outnum):
        """Set the output frequency of a PLL

        Args:
            freq (int): The desired output frequency of the PLL. Must be in
                range [630kHz, 167MHz]
            outnum (int): The PLL to set the output frequency of

        Raises:
            ValueError: Desired frequency is bigger than 167MHz or smaller than
                630kHz
        """
        if freq is None or (freq < 630E3) or (freq > 167E6):
            raise ValueError("Illegal clock frequency = %d" % freq)
            return
        best = self.calcMulDiv(freq, self.reffreq)
        self.pllwrite(outnum, N=best[0], M=best[1], outdiv=best[2])
        self.outputUpdateOutputs(outnum)

    def outnumToPin(self, outnum):
        """Convert from PLL Number to actual output pin"""
        if outnum == 0:
            return 0
        elif outnum == 1:
            return 1
        elif outnum == 2:
            return 4
        else:
            raise ValueError("Invalid output number = %d" % outnum)

    def outputUpdateOutputs(self, outnum, pllsrc_new=None, pllenabled_new=None, pllslewrate_new=None):
        """Update the output pins with enabled/disabled, slew rate, etc"""
        # Map to output pins on CDCE906 Chip
        if outnum == 0:
            outpin = 0
            if pllsrc_new is None:
                src = self._pll0source
            else:
                src = pllsrc_new

            if src == 'PLL0':
                divsrc = 0
            elif src == 'PLL1':
                divsrc = 1
            elif src == 'PLL2':
                divsrc = 2
        elif outnum == 1:
            outpin = 1
            divsrc = 1
        elif outnum == 2:
            outpin = 4
            divsrc = 2

        if pllenabled_new is None:
            pll_enabled = self.pll_outenable_get(outnum)
        else:
            pll_enabled = pllenabled_new

        if pllslewrate_new is None:
            if   (outnum == 0): pll_slewrate = self._pll0slew
            elif (outnum == 1): pll_slewrate = self._pll1slew
            elif (outnum == 2): pll_slewrate = self._pll2slew
        else:
            pll_slewrate = pllslewrate_new

        self.cdce906setoutput(outpin, divsrc, slewrate=pll_slewrate, enabled=pll_enabled)

    def pll_outfreq_get(self, outnum):
        """Read the programmed output frequency from a PLL on the CW305

        Args:
            outnum (int): PLL to read from (i.e. for PLL 0, use outnum=0)

        Returns:
            The output frequency of the specified PLL
        """
        settings = self.pllread(outnum)
        freq = ((self.reffreq * float(settings[0])) / float(settings[1])) / float(settings[2])
        return freq

    def pll_outenable_set(self, enabled, outnum):
        """Enable or disable a PLL

        Args:
            enabled (bool): Whether to enable (True) or disable (False) the
                specified PLL.
            outnum (int): The PLL to enable or disable
        """
        self.outputUpdateOutputs(outnum, pllenabled_new=enabled)

    def pll_outenable_get(self, outnum):
        """Get whether a PLL is enabled or not

        Args:
            outnum (int): The PLL to get the enable status of

        Returns:
            True if the PLL is enabled, False if it isn't
        """
        outpin = self.outnumToPin(outnum)
        data = self.cdce906read(19 + outpin)
        return bool(data & (1 << 3))

    def pll_outslew_set(self, slew, outnum):
        """ Set clock slew rate for the selected clock output.
            - outnum=0: specifies CLK-SMA X6 output.
            - outnum=1: specifies FPGA pin N13 output.
            - outnum=2: specifies FPGA pin E12 output.
        Allowed values: '+3nS', '+2nS', '+1nS', '+0nS'.
        """
        if slew in ['+3nS', '+2nS', '+1nS', '+0nS']:
            if   outnum == 0: self._pll0slew = slew
            elif outnum == 1: self._pll1slew = slew
            elif outnum == 2: self._pll2slew = slew
            else:
                raise ValueError("Invalid clock number: %d" % outnum)
            self.outputUpdateOutputs(outnum, pllslewrate_new=slew)
        else:
            raise ValueError("Invalid slew value: %s" % slew)

    def pll_outslew_get(self, outnum):
        """Get slew rate of PLL output"""
        outpin = self.outnumToPin(outnum)
        data = self.cdce906read(19 + outpin)
        slew = data & (3 << 4)

        if slew == (0 << 4):
            return "+3nS"
        elif slew == (1 << 4):
            return "+2nS"
        elif slew == (2 << 4):
            return "+1nS"
        else:
            return "+0nS"

    def pll_outsource_set(self, source, outnum):
        """Update clock source for the selected clock output.
            - outnum=0: specifies CLK-SMA X6 source; allowed source values: 'PLL0', 'PLL1', 'PLL2'.
            - outnum=1: specifies FPGA pin N13 source; allowed source values: 'PLL1'.
            - outnum=2: specifies FPGA pin E12 source; allowed source values: 'PLL2'.
        """
        if outnum==0 and source in ['PLL0', 'PLL1', 'PLL2']:
            self._pll0source = source
            self.outputUpdateOutputs(outnum, pllsrc_new=source)
        elif (outnum==1 and source == 'PLL1') or (outnum==2 and source == 'PLL2'):
            self.outputUpdateOutputs(outnum, pllsrc_new=source)
        else:
            raise ValueError("Invalid source %s for clock %d" % (source, outnum))

    def pll_outsource_get(self, outnum):
        """Get output source settings"""
        outpin = self.outnumToPin(outnum)
        data = self.cdce906read(19 + outpin)
        divsrc = data & 0x07

        if divsrc == 0:
            return "PLL0"
        elif divsrc == 1:
            return "PLL1"
        elif divsrc == 2:
            return "PLL2"
        else:
            return "UNKNOWN"

    def pll_writedefaults(self):
        """Save PLL settings to EEPROM, making them power-on defaults"""
        # Set bit high to enable write
        data = self.cdce906read(26) & (~(1 << 7))
        self.cdce906write(26, data | (1 << 7))

        # Wait until done
        while (self.cdce906read(24) & (1 << 7)):
            time.sleep(0.05)

        # Set bit low
        self.cdce906write(26, data)

    def cdce906write(self, addr, data):
        """ Write a byte to the CDCE906 External PLL Chip """

        # print "Write %d = %x" % (addr, data)
        self._usb.sendCtrl(0x30, data=[0x01, addr, data])
        resp = self._usb.readCtrl(0x30, dlen=2)
        if resp[0] != 2:
            raise IOError("CDCE906 Write Error, response = %d" % resp[0])

    def cdce906read(self, addr):
        """ Read a byte from the CDCE906 External PLL Chip """
        self._usb.sendCtrl(0x30, data=[0x00, addr, 0])
        resp = self._usb.readCtrl(0x30, dlen=2)
        if resp[0] != 2:
            raise IOError("CDCE906 Read Error, response = %d" % resp[0])
        return resp[1]

    def cdce906init(self):
        """ Set defaults on CDCE906 PLL Chip """
        # Default is:
        # Y0 = P0
        # Y1 = P1
        # Y2,Y3,Y5 = disabled
        # Y4 = P2
        self.outputUpdateOutputs(0)
        self.outputUpdateOutputs(1)
        self.outputUpdateOutputs(2)

        self.cdce906setoutput(2, 0, enabled=False)
        self.cdce906setoutput(3, 0, enabled=False)
        self.cdce906setoutput(5, 0, enabled=False)

        self.cdce906write(9, (1 << 5) | (self.cdce906read(9) & 0x1F))  # PLL 1
        self.cdce906write(10, (2 << 5) | (self.cdce906read(10) & 0x1F))  # PLL2 not SSC
        self.cdce906write(11, 4 | (self.cdce906read(11) & 0xF8))  # PLL 3

    def cdce906setoutput(self, outpin, divsource, slewrate='+0nS', enabled=True, inverted=False):
        """Setup the outputs for the PLL chip"""
        data = 0
        if enabled:
            data |= 1 << 3
        data |= divsource
        if slewrate == "+0nS":
            data |= (3 << 4)
        elif slewrate == "+1nS":
            data |= (2 << 4)
        elif slewrate == "+2nS":
            data |= (1 << 4)
        elif slewrate == "+3nS":
            data |= (0 << 4)
        else:
            raise ValueError("Invalid slew rate: %s" % str(slewrate))
        if inverted:
            data |= 1 << 6
        self.cdce906write(19 + outpin, data)

    def pll_enable_set(self, enabled):
        """Enable or disable the PLL chip"""
        base = self.cdce906read(12)
        if enabled:
            self.cdce906write(12, base & ~(1 << 6))
        else:
            self.cdce906write(12, base | (1 << 6))

    def pll_enable_get(self):
        """Read if PLL chip is enabled or disabled"""
        base = self.cdce906read(12)
        if base & (1 << 6):
            return False
        else:
            return True

    def calcMulDiv(self, freqdesired, freqsource):
        """Calculate Multiply & Divide settings for PLL based on input frequency"""

        lowerror = 1E99
        best = (0, 0, 0)

        # Figured out divider settings to put fvco in range 80-300
        maxoutdiv = int(math.floor(300E6 / freqdesired))
        minoutdiv = int(math.ceil(80E6 / freqdesired))
        maxoutdiv = min(maxoutdiv, 127)

        # If finds an exact match, this is very fast... if not this
        # can take a little while to calculate
        for outdiv in range(minoutdiv, maxoutdiv + 1):
            fvco = freqdesired * outdiv
            for N in range(1, 4096):
                for M in range(1, 512):
                    err = abs(fvco - ((freqsource * N) / M))
                    if err < lowerror:
                        lowerror = err / outdiv
                        best = (N, M, outdiv)
                        if lowerror == 0:
                            return best
        return best

    def pllwrite(self, pllnum, N, M, outdiv):
        """Write N/M/output divisors to PLL chip"""
        offset = 3 * pllnum

        self.cdce906write(1 + offset, M & 0xFF)
        self.cdce906write(2 + offset, N & 0xFF)
        base = self.cdce906read(3 + offset)
        base = base & 0xE0  # Mask out lower bits
        base |= (M & (0x100)) >> 8
        base |= (N & (0xF00)) >> 7
        self.cdce906write(3 + offset, base)
        self.cdce906write(13 + pllnum, outdiv & 0x7F)

        # Set PLL Mode (high-speed or regular)
        fvco = (self.reffreq * float(N)) / float(M)
        if fvco < 80E6 or fvco > 300E6:
            raise IOError("Fvco set to $d, out of range" % fvco)

        data = self.cdce906read(6)
        if pllnum == 0:
            pllbit = 7
        elif pllnum == 1:
            pllbit = 6
        elif pllnum == 2:
            pllbit = 5

        data &= ~(1 << pllbit)
        if fvco > 180E6:
            # High-speed mode (180-300 MHz)
            data |= 1 << pllbit
        self.cdce906write(6, data)

    def pllread(self, pllnum):
        """Read N/M/output divisors from PLL chip"""
        offset = 3 * pllnum

        # Read M & N
        M = self.cdce906read(1 + offset)
        N = self.cdce906read(2 + offset)
        temp = self.cdce906read(3 + offset)
        M |= ((temp & 0x01)) << 8
        N |= (temp & 0x1E) << 7

        outdiv = self.cdce906read(13 + pllnum) & 0x7F

        return (N, M, outdiv)



# Authors: Colin O'Flynn
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_DEBUG
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

raise DeprecationWarning("ChipWhisperer Rev2 is no longer supported")

import logging
import sys
# import chipwhisperer.capture.scopes._qt as openadc_qt
from .. import _qt as openadc_qt
from chipwhisperer.capture.scopes.cwhardware.ChipWhispererFWLoader import CWCRev2_Loader
from chipwhisperer.capture.scopes.cwhardware.ChipWhispererFWLoader import FWLoaderConfig
from chipwhisperer.capture.scopes.cwhardware.ChipWhispererFWLoaderGUI import FWLoaderConfigGUI
from chipwhisperer.common.utils.parameter import Parameterized

try:
    import usb
except ImportError:
    usb = None


class OpenADCInterface_ZTEX(Parameterized):
    _name = "ChipWhisperer Rev2"

    def __init__(self, oadcInstance):
        self.getParams().addChildren([
            {'name':'CW Firmware Preferences','tip':'Configure ChipWhisperer FW Paths', 'type':"menu", "action":lambda _:self.getFwLoaderConfigGUI.show()},
            {'name':'Download CW Firmware','tip':'Download Firmware+FPGA To Hardware', 'type':"menu", "action":lambda _:self.getCwFirmwareConfig.loadRequired()},
        ])
        self.ser = None

        if (openadc_qt is None) or (usb is None):
            missingInfo = ""
            if openadc_qt is None:
                missingInfo += "openadc.qt "
            if usb is None:
                missingInfo += " usb"
            raise ImportError("Needed imports for ChipWhisperer missing: %s" % missingInfo)
        else:
            self.scope = oadcInstance
            self.cwFirmwareConfig = FWLoaderConfig(CWCRev2_Loader())

    def con(self):
        if self.ser is None:
            # Download firmware if required
            self.cwFirmwareConfig.loadRequired()

            try:
                dev = usb.core.find(idVendor=0x221A, idProduct=0x0100)
            except IOError as e:
                exctype, value = sys.exc_info()[:2]
                raise IOError("FX2 Port " +  str(exctype) + str(value))

            if dev is None:
                raise IOError("FX2 Port. Could not open USB Device")

            dev.set_configuration()

            self.dev = dev
            self.writeEP = 0x06
            self.readEP = 0x82
            self.interface = 0
            self.ser = self

        try:
            self.scope.con(self.ser)
            logging.info('OpenADC Found, Connecting')
        except IOError as e:
            exctype, value = sys.exc_info()[:2]
            raise IOError("OpenADC Error (FX2 Port): " + (str(exctype) + str(value)) + " - Did you download firmware/FPGA data to ChipWhisperer?")

    def dis(self):
        self.ser = None

    def __del__(self):
        pass

    def read(self, N=0, debug=False):
        try:
            # self.interface removed from call for latest API compatibility
            data = self.dev.read(self.readEP, N, timeout=100)
        except IOError:
            return []

        data = bytearray(data)
        if debug:
            print("RX: ", end=' ')
            for b in data:
                print("%02x "%b, end=' ')
            print("")
        return data

    def write(self, data, debug=False):
        data = bytearray(data)
        if debug:
            print("TX: ", end=' ')
            for b in data:
                print("%02x "%b, end=' ')
            print("")
        # self.interface removed from call for latest API compatibility
        self.dev.write(self.writeEP, data, timeout=500)

    def getFwLoaderConfigGUI(self):
        if not hasattr(self, 'fwLoaderConfigGUI'):
            self.fwLoaderConfigGUI = FWLoaderConfigGUI(self.cwFirmwareConfig)
        return self.fwLoaderConfigGUI



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import ast
import collections
import os.path
import shutil
import weakref
import numpy as np
from functools import wraps
import warnings

try:
    # OrderedDict is new in 2.7
    from collections import OrderedDict
    DictType = OrderedDict
except ImportError:
    DictType = dict


def getRootDir():
    path = os.path.join(os.path.dirname(__file__), "../../../")
    return os.path.normpath(path)


def copyFile(source, destination, keepOriginals = True):
    if keepOriginals:
        shutil.copy2(source, destination)
    else:
        shutil.move(source, destination)


def strippedName(fullFileName):
    (filepath, filename) = os.path.split(fullFileName)
    (base, toplevel) = os.path.split(filepath)
    return toplevel + "/" + filename


def appendAndForwardErrorMessage(msg, e):
    raise type(e)(msg + "\n  -> " + str(e))


def list2hexstr(data, delim='', prefix=''):
    """
    Convert a list of integers to a hex string, with optional deliminators/prefix

    delim is inserted between each list item

    prefix is inserted infront of each item (including first item)
    """

    rstr = ["%02x" % t for t in data]
    rstr = (delim + prefix).join(rstr)
    rstr = prefix + rstr
    return rstr


def hexstr2list(data):
    """Convert a string with hex numbers into a list of numbers"""

    data = str(data)

    newdata = data.lower()
    newdata = newdata.replace("0x", "")
    newdata = newdata.replace(",", "")
    newdata = newdata.replace(" ", "")
    newdata = newdata.replace("[", "")
    newdata = newdata.replace("]", "")
    newdata = newdata.replace("(", "")
    newdata = newdata.replace(")", "")
    newdata = newdata.replace("{", "")
    newdata = newdata.replace("}", "")
    newdata = newdata.replace(":", "")
    newdata = newdata.replace("-", "")

    datalist = [int(newdata[i:(i + 2)], 16) for i in range(0, len(newdata), 2)]

    return datalist


def strListToList(strlist):
    """
    Convert string in form of '"[33, 42, 43]", "[24, 43, 4]"'
    into a normal list.
    """

    strlist = strlist.replace('"', '')
    strlist = strlist.replace("'", "")
    try:
        listeval = ast.literal_eval(strlist)
        return listeval
    except ValueError:
        raise ValueError("Failed to convert %s to list" % (strlist))


def convert_to_str(data):
    """
    Converts all dictionary elements to string type - similar to what ConfigObj will
    be doing when it saves and loads the data.
    """
    if isinstance(data, collections.Mapping):
        return dict(list(map(convert_to_str, iter(list(data.items())))))
    elif isinstance(data, collections.Iterable) and not isinstance(data, str):
        return type(data)(list(map(convert_to_str, data)))
    else:
        return str(data)


def hexStrToByteArray(hexStr):
    ba = bytearray(hexstr2list(hexStr))
    return ba


def binarylist2bytearray(bitlist, nrBits=8):
    ret = []
    pos = 0
    while pos <= len(bitlist) - nrBits:
        out = 0
        for bit in range(nrBits):
            out = (out << 1) | bitlist[pos + bit]
        ret.append(out)
        pos += nrBits
    return ret


def bytearray2binarylist(bytes, nrBits=8):
    init = np.array([], dtype=bool)
    for byte in bytes:
        init = np.concatenate((init, np.unpackbits(np.uint8(byte))[8 - nrBits:]), axis=0)
    return init


def getPyFiles(dir, extension=False):
    scriptList = []
    if os.path.isdir(dir):
        for fn in os.listdir(dir):
            fnfull = dir + '/' + fn
            if os.path.isfile(fnfull) and fnfull.lower().endswith('.py') and (not fnfull.endswith('__init__.py')) and (not fn.startswith('_')):
                if extension:
                    scriptList.append(fn)
                else:
                    scriptList.append(os.path.splitext(fn)[0])
    return scriptList

def _make_id(target):
    if hasattr(target, '__func__'):
        return (id(target.__self__))
    return id(target)


class Signal(object):
    class Cleanup(object):
        def __init__(self, key, d):
            self.key = key
            self.d = d

        def __call__(self, wr):
            del self.d[self.key]

    def __init__(self):
        self.callbacks = {}  #observing object ID -> weak ref, methodNames

    def connect(self, observer):
        if not callable(observer):
            raise TypeError('Expected a method, got %s' % observer.__class__)

        ID = _make_id(observer)
        if ID in self.callbacks:
            s = self.callbacks[ID][1]
        else:
            try:
                target = weakref.ref(observer.__self__, Signal.Cleanup(ID, self.callbacks))
            except AttributeError:
                target = None
            s = set()
            self.callbacks[ID] = (target, s)

        if hasattr(observer, "__func__"):
            method = observer.__func__
        else:
            method = observer
        s.add(method)

    def disconnect(self, observer):
        ID = _make_id(observer)
        if ID in self.callbacks:
            if hasattr(observer, "__func__"):
                method = observer.__func__
            else:
                method = observer
            self.callbacks[ID][1].discard(method)
            if len(self.callbacks[ID][1]) == 0:
                del self.callbacks[ID]
        else:
            pass

    def disconnectAll(self):
        self.callbacks = {}  # observing object ID -> weak ref, methods

    def emit(self, *args, **kwargs):
        callbacks = list(self.callbacks.keys())
        for ID in callbacks:
            try:
                target, methods = self.callbacks[ID]
            except KeyError:
                continue
            for method in methods.copy():
                if target is None:  # Lambda or partial
                    method(*args, **kwargs)
                else:
                    targetObj = target()
                    if targetObj is not None:
                        method(targetObj, *args, **kwargs)


class Observable(Signal):
    def __init__(self, value):
        super(Observable, self).__init__()
        self.data = value

    def setValue(self, value):
        if value != self.data:
            self.data = value
            self.emit()

    def value(self):
        return self.data


_consoleBreakRequested = False
class ConsoleBreakException(BaseException):
    """Custom exception class. Raised when pressing ctrl-C in console.

    This inherits from BaseException so that the generic "Save project?" window
    doesn't catch it.
    """
    pass

def requestConsoleBreak():
    global _consoleBreakRequested
    _consoleBreakRequested = True

_uiupdateFunction = None

def setUIupdateFunction(func):
    global _uiupdateFunction
    _uiupdateFunction= func

def updateUI():
    if _uiupdateFunction:
        _uiupdateFunction()

    # If an event handler has asked for a console break, raise an exception now
    global _consoleBreakRequested
    if _consoleBreakRequested:
        _consoleBreakRequested = False
        raise ConsoleBreakException()

class WeakMethod(object):
    """A callable object. Takes one argument to init: 'object.method'.
    Once created, call this object -- MyWeakMethod() --
    and pass args/kwargs as you normally would.
    """
    def __init__(self, object_dot_method, callback=None):
        try:
            if callback is None:
                self.target = weakref.ref(object_dot_method.__self__)
            else:
                self.target = weakref.ref(object_dot_method.__self__, callback)
            self.method = object_dot_method.__func__
        except AttributeError:
            self.target = None
            self.method = object_dot_method

    def __call__(self, *args, **kwargs):
        """Call the method with args and kwargs as needed."""
        if self.is_dead():
            raise TypeError('Method called on dead object')
        if self.target is None:  # Lambda or partial
            return self.method(*args, **kwargs)
        else:
            return self.method(self.target(), *args, **kwargs)

    def is_dead(self):
        '''Returns True if the referenced callable was a bound method and
        the instance no longer exists. Otherwise, return False.
        '''
        return self.target is not None and self.target() is None


class Command:
    """Converts a method call with arguments to be ignored in a simple call with no/fixed arguments (replaces lambda)"""
    def __init__(self, callback, *args, **kwargs):
        self.callback = WeakMethod(callback)
        self.args = args
        self.kwargs = kwargs

    def __call__(self, *args, **kwargs):
        return self.callback(*self.args, **self.kwargs)

if __name__ == '__main__':
    class test(object):
        def m(self):
            print("here")

        def __del__(self):
            print("deleted")

    x = test()
    y = x.m
    x = None
    y()

class DisableNewAttr(object):
    """Provides an ability to disable setting new attributes in a class, useful to prevent typos.

    Usage:
    1. Make a class that inherits this class:
    >>> class MyClass(DisableNewAttr):
    >>>     # Your class definition here

    2. After setting up all attributes that your object needs, call disable_newattr():
    >>>     def __init__(self):
    >>>         self.my_attr = 123
    >>>         self.disable_newattr()

    3. Subclasses raise an AttributeError when trying to make a new attribute:
    >>> obj = MyClass()
    >>> #obj.my_new_attr = 456   <-- Raises AttributeError
    """

    def __init__(self):
        self.disable_newattr()

    def disable_newattr(self):
        self._new_attributes_disabled = True

    def enable_newattr(self):
        self._new_attributes_disabled = False

    def __setattr__(self, name, value):
        if hasattr(self, '_new_attributes_disabled') and self._new_attributes_disabled and not hasattr(self, name):  # would this create a new attribute?
            raise AttributeError("Attempt to set unknown attribute in %s"%self.__class__, name)
        super(DisableNewAttr, self).__setattr__(name, value)


def dict_to_str(input_dict, indent=""):
    """Turn a dictionary of attributes/status values into a pretty-printed
    string for use on the command line. Recursively pretty-prints dictionaries.

    This function is most useful with OrderedDicts as it keeps the same
    printing order.
    """

    # Find minimum width that fits all names
    min_width = 0
    for n in input_dict:
        min_width = max(min_width, len(str(n)))

    # Build string
    ret = ""
    for n in input_dict:
        if type(input_dict[n]) in (dict, OrderedDict):
            ret += indent + str(n) + ' = '
            ret += '\n' + dict_to_str(input_dict[n], indent+"    ")
        else:
            ret += indent + str(n).ljust(min_width) + ' = '
            ret += str(input_dict[n]) + '\n'

    return ret


class bytearray(bytearray):
    """bytearray with better repr and str methods.

    Overwrites the __repr__ and __str__ methods of the builtin bytearray class
    so it prints without trying to turn everything into ascii characters.

    It should be usable like the builtin bytearray class in all other regards:
    """

    def __repr__(self):
        return "CWbytearray(b'{}')".format(' '.join(['{:0>2}'.format(hex(c)[2:]) for c in self]))

    def __str__(self):
        return self.__repr__()


class NoneTypeScope(object):
    """Raises an intelligible error related to scope disconnect when any attribute is accessed
    """

    def __getattr__(self, item):
        raise AttributeError('Scope has not been connected')


class NoneTypeTarget(object):
    """Raises an intelligble error related to target disconnect when any attribute is accessed
    """
    def __getattr__(self, item):
        raise AttributeError('Target has not been connected')


def camel_case_deprecated(func):
    """Wrapper function to deprecate camel case functions.

    This is not a decorator, do not use it that way. This way of deprecating
    allows the changing of the function definition name and then using this
    wrapper to define the camel case function, including a usage warning. To
    use first refactor the camelCase function definition to snake case, then
    use the wrapper on the snake_case function and assign it to the camelCase
    function name. It is best shown with an example:
    Before:
    .. code::
        def fooBar():
            pass

    After:
    .. code::
        def foo_bar():
            pass

        fooBar = camel_case_deprecated(foo_bar)

    Advantages of this method include being able to change the camelCase function
    to snake_case right away and keeping backwards compatibility, as well as
    supporting arbitrary amount of arguments and keyword arguments and keeping
    docstrings in tact.

    Args:
        func: The now snake_case function

    Returns: The wrapped snake_case function which now raises a warning during
        usage.
    """

    def underscore_to_camelcase(value):
    # .. function author:: Dave Webb: Stack overflow

        def camelcase():
            yield str.lower
            while True:
                yield str.capitalize

        c = camelcase()
        return ''.join(next(c)(x) if x else '_' for x in value.split('_'))

    cc_func = underscore_to_camelcase(func.__name__)

    @wraps(func)
    def wrapper(*args, **kwargs):
        warnings.warn('{} function is deprecated use {} instead.'.format(cc_func, func.__name__))
        return func(*args, *kwargs)

    wrapper.__name__ = underscore_to_camelcase(func.__name__)
    wrapper.__doc__ = 'Deprecated: Use {} instead.'.format(func.__name__)
    return wrapper


def get_cw_type(sn=None):
    """ Gets the scope type of the connected ChipWhisperer

    If multiple connected, sn must be specified
    """
    from chipwhisperer.hardware.naeusb.naeusb import NAEUSB
    from chipwhisperer.capture import scopes
    possible_ids = [0xace0, 0xace2, 0xace3]

    cwusb = NAEUSB()
    possible_sn = cwusb.get_possible_devices(idProduct=possible_ids)
    name = ""
    if len(possible_sn) == 0:
        raise OSError("USB Device not found. Did you connect it first?")

    if (len(possible_sn) > 1):
        if sn is None:
            serial_numbers = []
            for d in possible_sn:
                serial_numbers.append("sn = {} ({})".format(str(d['sn']), str(d['product'])))
            raise Warning("Multiple chipwhisperers connected, but device and/or serial number not specified.\nDevices:\n{}".format(serial_numbers))
        else:
            for d in possible_sn:
                if d['sn'] == sn:
                    name = d['product']
    else:
        name = possible_sn[0]['product']

    #print(name)
    if (name == "ChipWhisperer Lite") or (name == "ChipWhisperer CW1200"):
        return scopes.OpenADC
    elif name == "ChipWhisperer Nano":
        return scopes.CWNano

import time
def better_delay(ms):
    t = time.perf_counter() + ms / 1000
    while time.perf_counter() < t:
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from chipwhisperer.common.utils.parameter import Parameterized


class ReaderTemplate(Parameterized):
    _name= 'Smartcard Reader'

    def __init__(self):
        self.getParams()
        # self.getParams().addChildren([
        #     {'name':'Example Parameter', 'type':'int', 'value':5, 'set':self.setSomething}
        # ])

    def setSomething(self):
        """Here you would send value to the reader hardware"""
        pass

    def sendAPDU(self, cla, ins, p1, p2, txdata=None, rxdatalen=0):
        """Send APDU to SmartCard, get Response"""
        pass

    def con(self, scope=None):
        """Connect to reader."""
        pass

    def close(self):
        """Close connection."""
        pass

    def flush(self):
        """Discard all input buffers"""
        pass

    def reset(self):
        """Reset card & save the ATR"""
        pass

    def getATR(self):
        """Get the ATR from the SmartCard. Reads a saved value, user reset() to actually reset card."""
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from chipwhisperer.capture.api.programmers import Programmer
from chipwhisperer.common.utils import util

try:
    from Crypto.Cipher import AES
except ImportError:
    AES = None


class TargetTemplate:
    _name = 'Target Connection'
    connectStatus=False

    def __init__(self):
        pass

    def setSomething(self):
        """Here you would send value to the reader hardware"""
        pass

    def __del__(self):
        """Close system if needed"""
        self.close()

    def getStatus(self):
        return self.connectStatus

    def dis(self):
        """Disconnect from target"""
        self.close()
        self.connectStatus = False


    def con(self, scope=None, **kwargs):
        """Connect to target"""
        Programmer.lastFlashedFile = "unknown"
        try:
            self.connectStatus = True
            self._con(scope, **kwargs)
        except:
            self.dis()
            raise


    def _con(self, scope=None):
        raise NotImplementedError

    def flush(self):
        """Flush input/output buffers"""
        pass

    def close(self):
        """Close target"""
        pass

    def init(self):
        """Init Hardware"""
        pass

    def reinit(self):
        pass

    def setModeEncrypt(self):
        """Set hardware to do encryption, if supported"""
        pass

    def setModeDecrypt(self):
        """Set hardware to do decryption, if supported"""
        pass

    def checkEncryptionKey(self, key):
        """System 'suggests' encryption key, and target modifies it if required because e.g. hardware has fixed key"""
        return key

    def checkPlaintext(self, text):
        """System suggests plaintext; target modifies as required"""
        return text

    def loadEncryptionKey(self, key):
        """Load desired encryption key"""
        self.key = key

    def loadInput(self, inputtext):
        """Load input plaintext"""
        self.input = inputtext

    def isDone(self):
        """If encryption takes some time after 'go' called, lets user poll if done"""
        return True

    def readOutput(self):
        """Read result"""
        raise NotImplementedError("Target \"" + self.getName() + "\" does not implement method " + self.__class__.__name__ + ".readOutput()")

    def go(self):
        """Do Encryption"""
        raise NotImplementedError("Target \"" + self.getName() + "\" does not implement method " + self.__class__.__name__ + ".go()")

    def keyLen(self):
        """Length of key system is using"""
        return 16

    def textLen(self):
        """Length of the plaintext used by the system"""
        return 16

    def getExpected(self):
        """Based on key & text get expected if known, otherwise returns None"""

        # e.g. for AES we can do this:
        if AES and hasattr(self, 'key') and hasattr(self, 'input') and self.input and self.key:
            cipher = AES.new(bytes(self.key), AES.MODE_ECB)
            ct = cipher.encrypt(bytes(self.input))
            ct = bytearray(ct)
            return ct
        else:
            return None

    def validateSettings(self):
        # return [("warn", "Target Module", "You can't use module \"" + self.getName() + "\"", "Specify other module", "57a3924d-3794-4ca6-9693-46a7b5243727")]
        return []

    def _dict_repr(self):
        raise NotImplementedError("Must define target-specific properties.")

    def __repr__(self):
        return util.dict_to_str(self._dict_repr())

    def __str__(self):
        return self.__repr__()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#==========================================================================


import time
import os
from .naeusb import packuint32

class USART(object):
    """
    USART Class communicates with NewAE USB Interface to read/write data over control endpoint.
    """

    CMD_USART0_DATA = 0x1A
    CMD_USART0_CONFIG = 0x1B

    USART_CMD_INIT = 0x0010
    USART_CMD_ENABLE = 0x0011
    USART_CMD_DISABLE = 0x0012
    USART_CMD_NUMWAIT = 0x0014

    def __init__(self, usb, timeout=200):
        """
        Set the USB communications instance.
        """

        self._usb = usb
        self.timeout = timeout

        self._baud = 38400
        self._stopbits = 1
        self._parity = "none"

    def init(self, baud=115200, stopbits=1, parity="none"):
        """
        Open the serial port, set baud rate, parity, etc.
        """

        self._baud = baud
        self._stopbits = stopbits
        self._parity = parity

        if stopbits == 1:
            stopbits = 0
        elif stopbits == 1.5:
            stopbits = 1
        elif stopbits == 2:
            stopbits = 2
        else:
            raise ValueError("Invalid stop-bit spec: %s" % str(stopbits))

        if parity == "none":
            parity = 0
        elif parity == "odd":
            parity = 1
        elif parity == "even":
            parity = 2
        elif parity == "mark":
            parity = 3
        elif parity == "space":
            parity = 4
        else:
            raise ValueError("Invalid parity spec: %s" % str(parity))

        cmdbuf = packuint32(baud)
        cmdbuf.append(stopbits)
        cmdbuf.append(parity)
        cmdbuf.append(8)  # Data bits

        self._usartTxCmd(self.USART_CMD_INIT, cmdbuf)
        self._usartTxCmd(self.USART_CMD_ENABLE)

    def write(self, data, slow=False):
        """
        Send data to serial port.
        """
        # print "%d: %s" % (len(data), str(data))

        try:
            data = bytearray(data)
        except TypeError:
            try:
                data = bytearray(data, 'latin-1')
            except TypeError:
                #Second type-error happens if input was already list?
                pass

        datasent = 0

        while datasent < len(data):
            datatosend = len(data) - datasent
            datatosend = min(datatosend, 58)
            self._usb.sendCtrl(self.CMD_USART0_DATA, 0, data[datasent:(datasent + datatosend)])
            datasent += datatosend

    def flush(self):
        """
        Flush all input buffers
        """
        inwait = self.inWaiting()
        while(inwait):
            self.read(inwait)
            inwait = self.inWaiting()

    def inWaiting(self):
        """
        Get number of bytes waiting to be read.
        """
        # print "Checking Waiting..."
        data = self._usartRxCmd(self.USART_CMD_NUMWAIT, dlen=4)
        # print data
        return data[0]

    def read(self, dlen=0, timeout=0):
        """
        Read data from input buffer, if 'dlen' is 0 everything present is read. If timeout is non-zero
        system will block for a while until data is present in buffer.
        """

        waiting = self.inWaiting()

        if dlen == 0:
            dlen = waiting

        if timeout == 0:
            timeout = self.timeout


        resp = []

        while dlen and (timeout * 10) > 0:
            if waiting > 0:
                newdata = self._usb.readCtrl(self.CMD_USART0_DATA, 0, min(waiting, dlen))
                resp.extend(newdata)
                dlen -= len(newdata)
            waiting = self.inWaiting()
            timeout -= 1

        return resp


    def _usartTxCmd(self, cmd, data=[]):
        """
        Send a command to the USART interface (internal function).
        """

        # windex selects interface
        self._usb.sendCtrl(self.CMD_USART0_CONFIG, cmd, data)

    def _usartRxCmd(self, cmd, dlen=1):
        """
        Read the result of some command (internal function).
        """
        # windex selects interface, set to 0
        return self._usb.readCtrl(self.CMD_USART0_CONFIG, cmd, dlen)



"""
Package containing all of the target types that the ChipWhisperer API can connect to:

Targets:
* SimpleSerial - Communication via CW Uart Pins
* CW305 - USB communication to CW305
* SAKURAG - Requires FTDI module, old, untested
* SASEBOGII - Requires FTDI module, old, untested
* SmartCard - Currently unavailable (tied to GUI)
"""
from .SimpleSerial import SimpleSerial
from .CW305 import CW305 #check works

try:
    from .SAKURAG import SakuraG #needs ftdi module
    from .SASEBOGII import SaseboGII
except ImportError:
    pass

# from .SmartCard import SmartCard #Tied to GUI
